@@ -5,26 +5,26 @@
 
 pub mod bigsi;
 pub mod linear;
+pub mod revindex;
 pub mod sbt;
 
-pub mod storage;
-
 pub mod search;
 
 use std::ops::Deref;
 use std::path::Path;
-use std::rc::Rc;
 
 use once_cell::sync::OnceCell;
 use serde::{Deserialize, Serialize};
 use typed_builder::TypedBuilder;
 
+use crate::errors::ReadDataError;
 use crate::index::sbt::{Node, SBT};
 use crate::index::search::{search_minhashes, search_minhashes_containment};
-use crate::index::storage::{ReadData, ReadDataError, Storage};
-use crate::signature::{Signature, SigsTrait};
+use crate::prelude::*;
+use crate::signature::SigsTrait;
 use crate::sketch::nodegraph::Nodegraph;
 use crate::sketch::Sketch;
+use crate::storage::{InnerStorage, Storage};
 use crate::Error;
 
 pub type MHBT = SBT<Node<Nodegraph>, Signature>;
@@ -98,17 +98,19 @@ pub trait Index<'a> {
 
     fn signature_refs(&self) -> Vec<&Self::Item>;
 
+    fn len(&self) -> usize {
+        self.signature_refs().len()
+    }
+
+    fn is_empty(&self) -> bool {
+        self.len() == 0
+    }
+
     /*
     fn iter_signatures(&self) -> Self::SignatureIterator;
     */
 }
 
-// TODO: split into two traits, Similarity and Containment?
-pub trait Comparable<O> {
-    fn similarity(&self, other: &O) -> f64;
-    fn containment(&self, other: &O) -> f64;
-}
-
 impl<'a, N, L> Comparable<L> for &'a N
 where
     N: Comparable<L>,
@@ -140,7 +142,7 @@ pub struct SigStore<T> {
     #[builder(setter(into))]
     metadata: String,
 
-    storage: Option<Rc<dyn Storage>>,
+    storage: Option<InnerStorage>,
 
     #[builder(setter(into), default)]
     data: OnceCell<T>,
