@@ -16,7 +16,6 @@ use std::fs::File;
 use std::hash::BuildHasherDefault;
 use std::io::{BufReader, Read};
 use std::path::{Path, PathBuf};
-use std::rc::Rc;
 
 use log::info;
 use nohash_hasher::NoHashHasher;
@@ -24,26 +23,18 @@ use once_cell::sync::OnceCell;
 use serde::{Deserialize, Serialize};
 use typed_builder::TypedBuilder;
 
-use crate::index::storage::{FSStorage, ReadData, Storage, StorageInfo, ToWriter};
 use crate::index::{Comparable, DatasetInfo, Index, SigStore};
-use crate::signature::Signature;
+use crate::prelude::*;
+use crate::storage::{FSStorage, InnerStorage, StorageInfo};
 use crate::Error;
 
-pub trait Update<O> {
-    fn update(&self, other: &mut O) -> Result<(), Error>;
-}
-
-pub trait FromFactory<N> {
-    fn factory(&self, name: &str) -> Result<N, Error>;
-}
-
 #[derive(TypedBuilder)]
 pub struct SBT<N, L> {
     #[builder(default = 2)]
     d: u32,
 
     #[builder(default, setter(into))]
-    storage: Option<Rc<dyn Storage>>,
+    storage: Option<InnerStorage>,
 
     #[builder(default = Factory::GraphFactory { args: (1, 100000.0, 4) })]
     factory: Factory,
@@ -87,7 +78,7 @@ where
         (0..u64::from(self.d)).map(|c| self.child(pos, c)).collect()
     }
 
-    pub fn storage(&self) -> Option<Rc<dyn Storage>> {
+    pub fn storage(&self) -> Option<InnerStorage> {
         self.storage.clone()
     }
 
@@ -156,7 +147,7 @@ where
             SBTInfo::V6(ref sbt) => (&sbt.storage.args).into(),
         };
         st.set_base(path.as_ref().to_str().unwrap());
-        let storage: Rc<dyn Storage> = Rc::new(st);
+        let storage = InnerStorage::new(st);
 
         let d = match sinfo {
             SBTInfo::V4(ref sbt) => sbt.d,
@@ -182,7 +173,7 @@ where
                                 .filename(l.filename)
                                 .name(l.name)
                                 .metadata(l.metadata)
-                                .storage(Some(Rc::clone(&storage)))
+                                .storage(Some(storage.clone()))
                                 .build(),
                         )
                     })
@@ -197,7 +188,7 @@ where
                                 .filename(l.filename)
                                 .name(l.name)
                                 .metadata(l.metadata)
-                                .storage(Some(Rc::clone(&storage)))
+                                .storage(Some(storage.clone()))
                                 .build(),
                         )
                     })
@@ -215,7 +206,7 @@ where
                                 .filename(l.filename)
                                 .name(l.name)
                                 .metadata(l.metadata)
-                                .storage(Some(Rc::clone(&storage)))
+                                .storage(Some(storage.clone()))
                                 .build(),
                         )
                     })
@@ -230,7 +221,7 @@ where
                                 .filename(l.filename)
                                 .name(l.name)
                                 .metadata(l.metadata)
-                                .storage(Some(Rc::clone(&storage)))
+                                .storage(Some(storage.clone()))
                                 .build(),
                         )
                     })
@@ -248,7 +239,7 @@ where
                                 .filename(l.filename.clone())
                                 .name(l.name.clone())
                                 .metadata(l.metadata.clone())
-                                .storage(Some(Rc::clone(&storage)))
+                                .storage(Some(storage.clone()))
                                 .build(),
                         )),
                         NodeInfoV4::Leaf(_) => None,
@@ -266,7 +257,7 @@ where
                                 .filename(l.filename)
                                 .name(l.name)
                                 .metadata(l.metadata)
-                                .storage(Some(Rc::clone(&storage)))
+                                .storage(Some(storage.clone()))
                                 .build(),
                         )),
                     })
@@ -279,7 +270,7 @@ where
         Ok(SBT {
             d,
             factory,
-            storage: Some(Rc::clone(&storage)),
+            storage: Some(storage),
             nodes,
             leaves,
         })
@@ -303,7 +294,7 @@ where
     pub fn save_file<P: AsRef<Path>>(
         &mut self,
         path: P,
-        storage: Option<Rc<dyn Storage>>,
+        storage: Option<InnerStorage>,
     ) -> Result<(), Error> {
         let ref_path = path.as_ref();
         let mut basename = ref_path.file_name().unwrap().to_str().unwrap().to_owned();
@@ -316,7 +307,7 @@ where
             Some(s) => s,
             None => {
                 let subdir = format!(".sbt.{}", basename);
-                Rc::new(FSStorage::new(location.to_str().unwrap(), &subdir))
+                InnerStorage::new(FSStorage::new(location.to_str().unwrap(), &subdir))
             }
         };
 
@@ -339,7 +330,7 @@ where
                     let _: &U = (*l).data().expect("Couldn't load data");
 
                     // set storage to new one
-                    l.storage = Some(Rc::clone(&storage));
+                    l.storage = Some(storage.clone());
 
                     let filename = (*l).save(&l.filename).unwrap();
                     let new_node = NodeInfo {
@@ -358,7 +349,7 @@ where
                     let _: &T = (*l).data().unwrap();
 
                     // set storage to new one
-                    l.storage = Some(Rc::clone(&storage));
+                    l.storage = Some(storage.clone());
 
                     // TODO: this should be l.md5sum(), not l.filename
                     let filename = (*l).save(&l.filename).unwrap();
@@ -475,7 +466,7 @@ where
                 // Case 2: parent is a node and has an empty child spot available
                 // (if there isn't an empty spot, it was already covered by case 1)
                 Entry::Occupied(mut pnode) => {
-                    dataset.update(&mut pnode.get_mut())?;
+                    dataset.update(pnode.get_mut())?;
                     self.leaves.entry(pos).or_insert_with(|| dataset.into());
                     final_pos = pos;
                 }
@@ -500,7 +491,7 @@ where
                 //TODO: use children for this node to update, instead of dragging
                 // dataset up to the root? It would be more generic, but this
                 // works for minhash, draff signatures and nodegraphs...
-                data.update(&mut pnode.get_mut())?;
+                data.update(pnode.get_mut())?;
             }
             parent_pos = ppos;
         }
@@ -566,7 +557,7 @@ pub struct Node<T> {
     metadata: HashMap<String, u64>,
 
     #[builder(default)]
-    storage: Option<Rc<dyn Storage>>,
+    storage: Option<InnerStorage>,
 
     #[builder(setter(into), default)]
     data: OnceCell<T>,
@@ -704,7 +695,7 @@ struct TreeNode<T> {
 
 pub fn scaffold<N>(
     mut datasets: Vec<SigStore<Signature>>,
-    storage: Option<Rc<dyn Storage>>,
+    storage: Option<InnerStorage>,
 ) -> SBT<Node<N>, Signature>
 where
     N: Clone + Default,
