@@ -11,8 +11,14 @@ from . import downsample
 from . import extract
 from . import filter
 from . import flatten
+from . import fileinfo
+from . import fileinfo as summarize
+from . import grep
 from . import kmers
+from . import check
+from . import collect
 from . import intersect
+from . import inflate
 from . import manifest
 from . import merge
 from . import rename
