@@ -1,11 +1,27 @@
 """concatenate signature files"""
 
+usage="""
+
+### `sourmash signature cat` - concatenate multiple signatures together
+
+Concatenate signature files.
+
+For example,
+
+sourmash signature cat file1.sig file2.sig -o all.sig
+
+will combine all signatures in `file1.sig` and `file2.sig` and put them
+in the file `all.sig`.
+
+"""
+
 from sourmash.cli.utils import (add_moltype_args, add_ksize_arg,
-                                add_picklist_args)
+                                add_picklist_args, add_pattern_args)
 
 
 def subparser(subparsers):
-    subparser = subparsers.add_parser('cat')
+    # working on this
+    subparser = subparsers.add_parser('cat', description=__doc__, usage=usage)
     subparser.add_argument('signatures', nargs='*')
     subparser.add_argument(
         '--from-file',
@@ -29,6 +45,7 @@ def subparser(subparsers):
     )
     add_ksize_arg(subparser, 31)
     add_moltype_args(subparser)
+    add_pattern_args(subparser)
     add_picklist_args(subparser)
 
 
