@@ -1,10 +1,23 @@
 """export a signature, e.g. to mash"""
 
+usage="""
+
+### `sourmash signature export` - export signatures to mash.
+
+Export signatures from sourmash format. Currently only supports
+mash dump format.
+
+For example,
+
+sourmash signature export filename.sig -o filename.sig.msh.json
+
+"""
+
 from sourmash.cli.utils import add_ksize_arg, add_moltype_args
 
 
 def subparser(subparsers):
-    subparser = subparsers.add_parser('export')
+    subparser = subparsers.add_parser('export', description=__doc__, usage=usage)
     subparser.add_argument('filename')
     subparser.add_argument(
         '-q', '--quiet', action='store_true',
