@@ -1,11 +1,28 @@
 """rename signature"""
 
+usage="""
+
+### `sourmash signature rename` - rename a signature
+
+Rename the display name for one or more signatures - this is the name
+output for matches in `compare`, `search`, `gather`, etc.
+
+For example,
+
+sourmash signature rename file1.sig "new name" -o renamed.sig
+
+will place a renamed copy of the hashes in `file1.sig` in the file
+`renamed.sig`. If you provide multiple signatures, all will be renamed
+to the same name.
+
+"""
+
 from sourmash.cli.utils import (add_moltype_args, add_ksize_arg,
-                                add_picklist_args)
+                                add_picklist_args, add_pattern_args)
 
 
 def subparser(subparsers):
-    subparser = subparsers.add_parser('rename')
+    subparser = subparsers.add_parser('rename', description=__doc__, usage=usage)
     subparser.add_argument('signatures', nargs='*')
     subparser.add_argument('name')
     subparser.add_argument(
@@ -31,6 +48,7 @@ def subparser(subparsers):
     )
     add_ksize_arg(subparser, 31)
     add_moltype_args(subparser)
+    add_pattern_args(subparser)
     add_picklist_args(subparser)
 
 
