@@ -39,25 +39,29 @@ class LCA_Database(Index):
     the `ident` keyword argument in `insert`.
 
     Integer `idx` indices can be used as keys in dictionary attributes:
-    * `idx_to_lid`, to get an (optional) lineage index.
-    * `idx_to_ident`, to retrieve the unique string identifier for that `idx`.
+    * `_idx_to_lid`, to get an (optional) lineage index.
+    * `_idx_to_ident`, to retrieve the unique string identifier for that `idx`.
 
     Integer `lid` indices can be used as keys in dictionary attributes:
-    * `lid_to_idx`, to get a set of `idx` with that lineage.
-    * `lid_to_lineage`, to get a lineage for that `lid`.
+    * `_lid_to_idx`, to get a set of `idx` with that lineage.
+    * `_lid_to_lineage`, to get a lineage for that `lid`.
 
-    `lineage_to_lid` is a dictionary with tuples of LineagePair as keys,
+    `_lineage_to_lid` is a dictionary with tuples of LineagePair as keys,
     `lid` as values.
 
-    `ident_to_name` is a dictionary from unique str identifer to a name.
+    `_ident_to_name` is a dictionary from unique str identifer to a name.
 
-    `ident_to_idx` is a dictionary from unique str identifer to integer `idx`.
+    `_ident_to_idx` is a dictionary from unique str identifer to integer `idx`.
 
-    `hashval_to_idx` is a dictionary from individual hash values to sets of
+    `_hashval_to_idx` is a dictionary from individual hash values to sets of
     `idx`.
     """
     is_database = True
 
+    # we set manifest to None to avoid implication of fast on-disk access to
+    # sketches. This may be revisited later.
+    manifest = None
+
     def __init__(self, ksize, scaled, moltype='DNA'):
         self.ksize = int(ksize)
         self.scaled = int(scaled)
@@ -66,25 +70,43 @@ class LCA_Database(Index):
 
         self._next_index = 0
         self._next_lid = 0
-        self.ident_to_name = {}
-        self.ident_to_idx = {}
-        self.idx_to_lid = {}
-        self.lineage_to_lid = {}
-        self.lid_to_lineage = {}
-        self.hashval_to_idx = defaultdict(set)
+        self._ident_to_name = {}
+        self._ident_to_idx = {}
+        self._idx_to_lid = {}
+        self._lineage_to_lid = {}
+        self._lid_to_lineage = {}
+        self._hashval_to_idx = defaultdict(set)
         self.picklists = []
 
     @property
     def location(self):
+        """Return source filename.
+
+        Part of the Index protocol.
+        """
         return self.filename
 
+    def __len__(self):
+        """Return number of sketches.
+
+        Part of the Index protocol.
+        """
+        return self._next_index
+
     def _invalidate_cache(self):
+        """Force rebuild of signatures after an 'insert'.
+
+        Internal method.
+        """
         if hasattr(self, '_cache'):
             del self._cache
 
     def _get_ident_index(self, ident, fail_on_duplicate=False):
-        "Get (create if nec) a unique int id, idx, for each identifier."
-        idx = self.ident_to_idx.get(ident)
+        """Get (create if necessary) a unique int idx, for each identifier.
+
+        Internal method.
+        """
+        idx = self._ident_to_idx.get(ident)
         if fail_on_duplicate:
             assert idx is None     # should be no duplicate identities
 
@@ -92,14 +114,18 @@ class LCA_Database(Index):
             idx = self._next_index
             self._next_index += 1
 
-            self.ident_to_idx[ident] = idx
+            self._ident_to_idx[ident] = idx
 
         return idx
 
     def _get_lineage_id(self, lineage):
-        "Get (create if nec) a unique lineage ID for each LineagePair tuples."
+        """Get (create if necessary) a unique lineage ID for each
+        LineagePair tuples."
+
+        Internal method of this class.
+        """
         # does one exist already?
-        lid = self.lineage_to_lid.get(lineage)
+        lid = self._lineage_to_lid.get(lineage)
 
         # nope - create one. Increment next_lid.
         if lid is None:
@@ -107,8 +133,8 @@ class LCA_Database(Index):
             self._next_lid += 1
 
             # build mappings
-            self.lineage_to_lid[lineage] = lid
-            self.lid_to_lineage[lid] = lineage
+            self._lineage_to_lid[lineage] = lid
+            self._lid_to_lineage[lid] = lineage
 
         return lid
 
@@ -121,6 +147,8 @@ class LCA_Database(Index):
         if not specified, the signature name (sig.name) is used.
 
         'lineage', if specified, must contain a tuple of LineagePair objects.
+
+        Method unique to this class.
         """
         minhash = sig.minhash
 
@@ -140,14 +168,14 @@ class LCA_Database(Index):
         if not ident:
             ident = str(sig)
 
-        if ident in self.ident_to_name:
+        if ident in self._ident_to_name:
             raise ValueError("signature '{}' is already in this LCA db.".format(ident))
 
         # before adding, invalide any caching from @cached_property
         self._invalidate_cache()
 
         # store full name
-        self.ident_to_name[ident] = sig.name
+        self._ident_to_name[ident] = sig.name
 
         # identifier -> integer index (idx)
         idx = self._get_ident_index(ident, fail_on_duplicate=True)
@@ -159,12 +187,12 @@ class LCA_Database(Index):
                 lid = self._get_lineage_id(lineage)
 
                 # map idx to lid as well.
-                self.idx_to_lid[idx] = lid
+                self._idx_to_lid[idx] = lid
             except TypeError:
                 raise ValueError('lineage cannot be used as a key?!')
 
         for hashval in minhash.hashes:
-            self.hashval_to_idx[hashval].add(idx)
+            self._hashval_to_idx[hashval].add(idx)
 
         return len(minhash)
 
@@ -172,14 +200,33 @@ class LCA_Database(Index):
         return "LCA_Database('{}')".format(self.filename)
 
     def signatures(self):
-        "Return all of the signatures in this LCA database."
+        """Return all of the signatures in this LCA database.
+
+        Part of the Index protocol.
+        """
         from sourmash import SourmashSignature
-        for v in self._signatures.values():
-            yield v
+
+        if self.picklists:
+            pl = self.picklists
+            for v in self._signatures.values():
+                if passes_all_picklists(v, pl):
+                    yield v
+        else:
+            for v in self._signatures.values():
+                yield v
+
+    def _signatures_with_internal(self):
+        """Return all of the signatures in this LCA database.
+
+        Part of the Index protocol; used for buulding manifests.
+        """
+
+        for idx, ss in self._signatures.items():
+            yield ss, idx
 
     def select(self, ksize=None, moltype=None, num=0, scaled=0, abund=None,
                containment=False, picklist=None):
-        """Make sure this database matches the requested requirements.
+        """Select a subset of signatures to search.
 
         As with SBTs, queries with higher scaled values than the database
         can still be used for containment search, but not for similarity
@@ -211,12 +258,21 @@ class LCA_Database(Index):
 
     @classmethod
     def load(cls, db_name):
-        "Load LCA_Database from a JSON file."
+        """Load LCA_Database from a JSON file.
+
+        Method specific to this class.
+        """
         from .lca_utils import taxlist, LineagePair
 
         if not os.path.isfile(db_name):
             raise ValueError(f"'{db_name}' is not a file and cannot be loaded as an LCA database")
 
+        try:
+            from sourmash.index.sqlite_index import LCA_SqliteDatabase
+            return LCA_SqliteDatabase.load(db_name)
+        except ValueError:
+            pass
+
         xopen = open
         if db_name.endswith('.gz'):
             xopen = gzip.open
@@ -278,8 +334,8 @@ class LCA_Database(Index):
                 vv = tuple(vv)
                 lid_to_lineage[int(k)] = vv
                 lineage_to_lid[vv] = int(k)
-            db.lid_to_lineage = lid_to_lineage
-            db.lineage_to_lid = lineage_to_lid
+            db._lid_to_lineage = lid_to_lineage
+            db._lineage_to_lid = lineage_to_lid
 
             # convert hashval -> lineage index keys to integers (looks like
             # JSON doesn't have a 64 bit type so stores them as strings)
@@ -288,21 +344,43 @@ class LCA_Database(Index):
 
             for k, v in hashval_to_idx_2.items():
                 hashval_to_idx[int(k)] = v
-            db.hashval_to_idx = hashval_to_idx
+            db._hashval_to_idx = hashval_to_idx
 
-            db.ident_to_name = load_d['ident_to_name']
-            db.ident_to_idx = load_d['ident_to_idx']
+            db._ident_to_name = load_d['ident_to_name']
+            db._ident_to_idx = load_d['ident_to_idx']
 
-            db.idx_to_lid = {}
+            db._idx_to_lid = {}
             for k, v in load_d['idx_to_lid'].items():
-                db.idx_to_lid[int(k)] = v
+                db._idx_to_lid[int(k)] = v
+
+        if db._ident_to_idx:
+            db._next_index = max(db._ident_to_idx.values()) + 1
+        else:
+            db._next_index = 0
+        if db._idx_to_lid:
+            db._next_lid = max(db._idx_to_lid.values()) + 1
+        else:
+            db._next_lid = 0
 
         db.filename = db_name
 
         return db
 
-    def save(self, db_name):
-        "Save LCA_Database to a JSON file."
+    def save(self, db_name, *, format='json'):
+        if format == 'sql':
+            self.save_to_sql(db_name)
+        else:
+            assert format == 'json'
+            self.save_to_json(db_name)
+
+    def save_to_json(self, db_name):
+        """Save LCA_Database to a JSON file.
+
+        Method specific to this class.
+        """
+        if os.path.exists(db_name):
+            raise ValueError(f"LCA database {db_name} already exists; not overwriting or appending")
+
         xopen = open
         if db_name.endswith('.gz'):
             xopen = gzip.open
@@ -324,27 +402,53 @@ class LCA_Database(Index):
 
             # convert lineage internals from tuples to dictionaries
             d = OrderedDict()
-            for k, v in self.lid_to_lineage.items():
+            for k, v in self._lid_to_lineage.items():
                 d[k] = dict([ (vv.rank, vv.name) for vv in v ])
             save_d['lid_to_lineage'] = d
 
             # convert values from sets to lists, so that JSON knows how to save
             save_d['hashval_to_idx'] = \
-               dict((k, list(v)) for (k, v) in self.hashval_to_idx.items())
+               dict((k, list(v)) for (k, v) in self._hashval_to_idx.items())
 
-            save_d['ident_to_name'] = self.ident_to_name
-            save_d['ident_to_idx'] = self.ident_to_idx
-            save_d['idx_to_lid'] = self.idx_to_lid
-            save_d['lid_to_lineage'] = self.lid_to_lineage
+            save_d['ident_to_name'] = self._ident_to_name
+            save_d['ident_to_idx'] = self._ident_to_idx
+            save_d['idx_to_lid'] = self._idx_to_lid
+            save_d['lid_to_lineage'] = self._lid_to_lineage
             
             json.dump(save_d, fp)
 
+    def save_to_sql(self, dbname):
+        "Save this LCA_Database into an LCA_SqliteDatabase"
+        from sourmash.index.sqlite_index import LCA_SqliteDatabase
+        from sourmash.tax.tax_utils import LineageDB
+
+        if os.path.exists(dbname):
+            raise ValueError(f"LCA database {dbname} already exists; not overwriting or appending")
+
+        # create a new in-memory lineage db...
+        assignments = {}
+        available_ranks = set() # track ranks, too
+        for ident, idx in self._ident_to_idx.items():
+            lid = self._idx_to_lid.get(idx)
+            if lid is not None:
+                lineage = self._lid_to_lineage[lid]
+                assignments[ident] = lineage
+                for pair in lineage:
+                    available_ranks.add(pair.rank)
+
+        ldb = LineageDB(assignments, available_ranks)
+
+        # ...and pass over to create, using 'self' as index.
+        LCA_SqliteDatabase.create(dbname, self, ldb)
+
     def downsample_scaled(self, scaled):
         """
         Downsample to the provided scaled value, i.e. eliminate all hashes
         that don't fall in the required range.
 
         This applies to this database in place.
+
+        Method specific to LCA databases.
         """
         if scaled == self.scaled:
             return
@@ -356,31 +460,53 @@ class LCA_Database(Index):
         max_hash = _get_max_hash_for_scaled(scaled)
 
         # filter out all hashes over max_hash in value.
-        new_hashvals = {}
-        for k, v in self.hashval_to_idx.items():
+        new_hashvals = defaultdict(set)
+        for k, v in self._hashval_to_idx.items():
             if k < max_hash:
                 new_hashvals[k] = v
-        self.hashval_to_idx = new_hashvals
+        self._hashval_to_idx = new_hashvals
         self.scaled = scaled
 
-    def get_lineage_assignments(self, hashval):
-        """
-        Get a list of lineages for this hashval.
+    @property
+    def hashvals(self):
+        "Return all hashvals stored in this database."
+        return self._hashval_to_idx.keys()
+
+    def get_lineage_assignments(self, hashval, min_num=None):
+        """Get a list of lineages for this hashval.
+
+        Method specific to LCA Databases.
         """
         x = []
 
-        idx_list = self.hashval_to_idx.get(hashval, [])
+        idx_list = self._hashval_to_idx.get(hashval, [])
+
+        if min_num and len(idx_list) < min_num:
+            return []
+
         for idx in idx_list:
-            lid = self.idx_to_lid.get(idx, None)
+            lid = self._idx_to_lid.get(idx, None)
             if lid is not None:
-                lineage = self.lid_to_lineage[lid]
+                lineage = self._lid_to_lineage[lid]
                 x.append(lineage)
 
         return x
 
+    def get_identifiers_for_hashval(self, hashval):
+        """
+        Get a list of identifiers for signatures containing this hashval
+        """
+        idx_list = self._hashval_to_idx.get(hashval, [])
+
+        for idx in idx_list:
+            yield self._idx_to_ident[idx]
+
     @cached_property
     def _signatures(self):
-        "Create a _signatures member dictionary that contains {idx: sigobj}."
+        """Create a _signatures member dictionary that contains {idx: sigobj}.
+
+        Internal method of this class.
+        """
         from sourmash import MinHash, SourmashSignature
 
         is_protein = False
@@ -401,7 +527,7 @@ class LCA_Database(Index):
         temp_vals = defaultdict(list)
 
         # invert the hashval_to_idx dictionary
-        for (hashval, idlist) in self.hashval_to_idx.items():
+        for (hashval, idlist) in self._hashval_to_idx.items():
             for idx in idlist:
                 temp_hashes = temp_vals[idx]
                 temp_hashes.append(hashval)
@@ -424,12 +550,13 @@ class LCA_Database(Index):
 
         sigd = {}
         for idx, mh in mhd.items():
-            ident = self.idx_to_ident[idx]
-            name = self.ident_to_name[ident]
+            ident = self._idx_to_ident[idx]
+            name = self._ident_to_name[ident]
             ss = SourmashSignature(mh, name=name)
+            ss.into_frozen()
 
             if passes_all_picklists(ss, self.picklists):
-                sigd[idx] = SourmashSignature(mh, name=name)
+                sigd[idx] = ss
 
         debug('=> {} signatures!', len(sigd))
         return sigd
@@ -443,6 +570,8 @@ class LCA_Database(Index):
         As with SBTs, queries with higher scaled values than the database
         can still be used for containment search, but not for similarity
         search. See SBT.select(...) for details.
+
+        Part of the Index protocol.
         """
         search_fn.check_is_compatible(query)
 
@@ -460,7 +589,7 @@ class LCA_Database(Index):
         c = Counter()
         query_hashes = set(query_mh.hashes)
         for hashval in query_hashes:
-            idx_list = self.hashval_to_idx.get(hashval, [])
+            idx_list = self._hashval_to_idx.get(hashval, [])
             for idx in idx_list:
                 c[idx] += 1
 
@@ -489,26 +618,37 @@ class LCA_Database(Index):
             score = search_fn.score_fn(query_size, shared_size, subj_size,
                                        total_size)
 
-            # note to self: even with JaccardSearchBestOnly, this will
+            # CTB note to self: even with JaccardSearchBestOnly, this will
             # still iterate over & score all signatures. We should come
             # up with a protocol by which the JaccardSearch object can
             # signal that it is done, or something.
+            # For example, see test_lca_jaccard_ordering, where
+            # for containment we could be done early, but for Jaccard we
+            # cannot.
             if search_fn.passes(score):
                 if search_fn.collect(score, subj):
                     if passes_all_picklists(subj, self.picklists):
                         yield IndexSearchResult(score, subj, self.location)
 
     @cached_property
-    def lid_to_idx(self):
+    def _lid_to_idx(self):
+        """Connect lineage id lid (int) to idx set (set of ints).""
+
+        Method specific to LCA databases.
+        """
         d = defaultdict(set)
-        for idx, lid in self.idx_to_lid.items():
+        for idx, lid in self._idx_to_lid.items():
             d[lid].add(idx)
         return d
 
     @cached_property
-    def idx_to_ident(self):
+    def _idx_to_ident(self):
+        """Connect idx (int) to ident (str).
+
+        Method specific to LCA databases.
+        """
         d = defaultdict(set)
-        for ident, idx in self.ident_to_idx.items():
+        for ident, idx in self._ident_to_idx.items():
             assert idx not in d
             d[idx] = ident
         return d
@@ -531,7 +671,7 @@ def load_databases(filenames, scaled=None, verbose=True):
     for db_name in filenames:
         if verbose:
             notify(u'\r\033[K', end=u'')
-            notify('... loading database {}'.format(db_name), end='\r')
+            notify(f'... loading database {format(db_name)}', end='\r')
 
         lca_db = LCA_Database.load(db_name)
 
@@ -555,7 +695,6 @@ def load_databases(filenames, scaled=None, verbose=True):
 
     if verbose:
         notify(u'\r\033[K', end=u'')
-        notify('loaded {} LCA databases. ksize={}, scaled={} moltype={}',
-               len(dblist), ksize, scaled, moltype)
+        notify(f'loaded {len(dblist)} LCA databases. ksize={ksize}, scaled={scaled} moltype={moltype}')
 
     return dblist, ksize, scaled
