@@ -1,4 +1,4 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 """
 sourmash submodule that provides MinHash class and utility functions.
 
@@ -6,6 +6,11 @@ class MinHash - core MinHash class.
 class FrozenMinHash - read-only MinHash class.
 """
 from __future__ import unicode_literals, division
+from .distance_utils import jaccard_to_distance, containment_to_distance, set_size_chernoff
+from .logging import notify
+
+import numpy as np
+
 
 __all__ = ['get_minhash_default_seed',
            'get_minhash_max_hash',
@@ -98,6 +103,39 @@ def translate_codon(codon):
         raise ValueError(e.message)
 
 
+def flatten_and_downsample_scaled(mh, *scaled_vals):
+    "Flatten MinHash object and downsample to max of scaled values."
+    assert mh.scaled
+    assert all( (x > 0 for x in scaled_vals) )
+
+    mh = mh.flatten()
+    scaled = max(scaled_vals)
+    if scaled > mh.scaled:
+        return mh.downsample(scaled=scaled)
+    return mh
+
+
+def flatten_and_downsample_num(mh, *num_vals):
+    "Flatten MinHash object and downsample to min of num values."
+    assert mh.num
+    assert all( (x > 0 for x in num_vals) )
+
+    mh = mh.flatten()
+    num = min(num_vals)
+    if num < mh.num:
+        return mh.downsample(num=num)
+    return mh
+
+
+def flatten_and_intersect_scaled(mh1, mh2):
+    "Flatten and downsample two scaled MinHash objs, then return intersection."
+    scaled = max(mh1.scaled, mh2.scaled)
+    mh1 = mh1.flatten().downsample(scaled=scaled)
+    mh2 = mh2.flatten().downsample(scaled=scaled)
+
+    return mh1 & mh2
+
+
 class _HashesWrapper(Mapping):
     "A read-only view of the hashes contained by a MinHash object."
     def __init__(self, h):
@@ -154,6 +192,7 @@ class MinHash(RustObject):
         self,
         n,
         ksize,
+        *,
         is_protein=False,
         dayhoff=False,
         hp=False,
@@ -236,9 +275,13 @@ class MinHash(RustObject):
 
     def __getstate__(self):
         "support pickling via __getstate__/__setstate__"
+
+        # note: we multiple ksize by 3 here so that
+        # pickle protocols that bypass __setstate__ <coff numpy coff>
+        # get a ksize that makes sense to the Rust layer. See #2262.
         return (
             self.num,
-            self.ksize,
+            self.ksize if self.is_dna else self.ksize*3,
             self.is_protein,
             self.dayhoff,
             self.hp,
@@ -281,12 +324,12 @@ class MinHash(RustObject):
         a = MinHash(
             self.num,
             self.ksize,
-            self.is_protein,
-            self.dayhoff,
-            self.hp,
-            self.track_abundance,
-            self.seed,
-            self._max_hash,
+            is_protein=self.is_protein,
+            dayhoff=self.dayhoff,
+            hp=self.hp,
+            track_abundance=self.track_abundance,
+            seed=self.seed,
+            max_hash=self._max_hash,
         )
         return a
 
@@ -615,8 +658,10 @@ class MinHash(RustObject):
 
         # end checks! create new object:
         a = MinHash(
-            num, self.ksize, self.is_protein, self.dayhoff, self.hp,
-            self.track_abundance, self.seed, max_hash
+            num, self.ksize,
+            is_protein=self.is_protein, dayhoff=self.dayhoff, hp=self.hp,
+            track_abundance=self.track_abundance, seed=self.seed,
+            max_hash=max_hash
         )
         # copy over hashes:
         if self.track_abundance:
@@ -631,8 +676,9 @@ class MinHash(RustObject):
         if self.track_abundance:
             # create new object:
             a = MinHash(
-                self.num, self.ksize, self.is_protein, self.dayhoff, self.hp,
-                False, self.seed, self._max_hash
+                self.num, self.ksize,
+                is_protein=self.is_protein, dayhoff=self.dayhoff, hp=self.hp,
+                track_abundance=False, seed=self.seed, max_hash=self._max_hash
             )
             a.add_many(self)
 
@@ -646,6 +692,30 @@ class MinHash(RustObject):
             raise TypeError(err)
         return self._methodcall(lib.kmerminhash_similarity, other._get_objptr(), True, downsample)
 
+    def jaccard_ani(self, other, *, downsample=False, jaccard=None, prob_threshold=1e-3,  err_threshold=1e-4):
+        "Use jaccard to estimate ANI between two MinHash objects."
+        if not (self.scaled and other.scaled):
+            raise TypeError("Error: can only calculate ANI for scaled MinHashes")
+        self_mh = self
+        other_mh = other
+        scaled = self.scaled
+        if downsample:
+            scaled = max(self_mh.scaled, other_mh.scaled)
+            self_mh = self.downsample(scaled=scaled)
+            other_mh = other.downsample(scaled=scaled)
+        if jaccard is None:
+            jaccard = self_mh.similarity(other_mh, ignore_abundance=True)
+        avg_sketch_kmers = (len(self_mh) + len(other_mh))/2
+        avg_n_kmers = round(avg_sketch_kmers * scaled)  # would be better if hll estimate - see #1798
+        j_aniresult = jaccard_to_distance(jaccard, self_mh.ksize, scaled,
+                                          n_unique_kmers=avg_n_kmers,
+                                          prob_threshold = prob_threshold,
+                                          err_threshold = err_threshold)
+        # null out ANI if either mh size estimation is inaccurate
+        if not self.size_is_accurate() or not other.size_is_accurate():
+            j_aniresult.size_is_inaccurate = True
+        return j_aniresult
+
     def similarity(self, other, ignore_abundance=False, downsample=False):
         """Calculate similarity of two sketches.
 
@@ -666,6 +736,8 @@ class MinHash(RustObject):
 
     def angular_similarity(self, other):
         "Calculate the angular similarity."
+        if not (self.track_abundance and other.track_abundance):
+            raise TypeError("Error: Angular (cosine) similarity requires both sketches to track hash abundance.")
         return self._methodcall(lib.kmerminhash_angular_similarity,
                                 other._get_objptr())
 
@@ -673,28 +745,158 @@ class MinHash(RustObject):
         return self._methodcall(lib.kmerminhash_is_compatible, other._get_objptr())
 
     def contained_by(self, other, downsample=False):
-        """\
+        """
         Calculate how much of self is contained by other.
         """
         if not (self.scaled and other.scaled):
-            raise TypeError("can only calculate containment for scaled MinHashes")
+            raise TypeError("Error: can only calculate containment for scaled MinHashes")
         if not len(self):
             return 0.0
-
         return self.count_common(other, downsample) / len(self)
+        # with bias factor
+        #return self.count_common(other, downsample) / (len(self) * (1- (1-1/self.scaled)^(len(self)*self.scaled)))
+
+
+    def contained_by_debiased(self, other, downsample=False):
+        """
+        Calculate how much of self is contained by other.
+        """
+        if not (self.scaled and other.scaled):
+            raise TypeError("Error: can only calculate containment for scaled MinHashes")
+        denom = len(self)
+        if not denom:
+            return 0.0
+        total_denom = float(denom * self.scaled) # would be better if hll estimate - see #1798
+        bias_factor = 1.0 - (1.0 - 1.0/self.scaled) ** total_denom
+        containment = self.count_common(other, downsample) / (denom * bias_factor)
+        # debiasing containment can lead to vals outside of 0-1 range!?
+        if containment >= 1:
+            return 1
+        elif containment <= 0:
+            return 0
+        else:
+            return containment
+
+
+    def containment_ani(self, other, *, downsample=False, containment=None, confidence=0.95, estimate_ci = False, prob_threshold=1e-3):
+        "Use self contained by other to estimate ANI between two MinHash objects."
+        if not (self.scaled and other.scaled):
+            raise TypeError("Error: can only calculate ANI for scaled MinHashes")
+        self_mh = self
+        other_mh = other
+        scaled = self.scaled
+        if downsample:
+            scaled = max(self_mh.scaled, other_mh.scaled)
+            self_mh = self.downsample(scaled=scaled)
+            other_mh = other.downsample(scaled=scaled)
+        containment = self_mh.contained_by_debiased(other_mh) # recalc debiased containment
+        #containment = self_mh.contained_by(other_mh) # recalc debiased containment
+        n_kmers = len(self_mh) * scaled # would be better if hll estimate - see #1798
+
+        c_aniresult = containment_to_distance(containment, self_mh.ksize, self_mh.scaled,
+                                                        n_unique_kmers=n_kmers, confidence=confidence,
+                                                        estimate_ci = estimate_ci, prob_threshold=prob_threshold)
+        # null out ANI if either mh size estimation is inaccurate
+        if not self.size_is_accurate() or not other.size_is_accurate():
+            c_aniresult.size_is_inaccurate = True
+        return c_aniresult
+
 
     def max_containment(self, other, downsample=False):
         """
         Calculate maximum containment.
         """
         if not (self.scaled and other.scaled):
-            raise TypeError("can only calculate containment for scaled MinHashes")
+            raise TypeError("Error: can only calculate containment for scaled MinHashes")
         min_denom = min((len(self), len(other)))
         if not min_denom:
             return 0.0
 
         return self.count_common(other, downsample) / min_denom
 
+
+    def max_containment_debiased(self, other, downsample=False):
+        """
+        Calculate maximum containment.
+        """
+        if not (self.scaled and other.scaled):
+            raise TypeError("Error: can only calculate containment for scaled MinHashes")
+        min_denom = min((len(self), len(other)))
+        if not min_denom:
+            return 0.0
+        total_denom =  float(min_denom * self.scaled) # would be better if hll estimate - see #1798
+        bias_factor = 1.0 - (1.0 - 1.0/self.scaled) ** total_denom
+        max_containment = self.count_common(other, downsample) / (min_denom * bias_factor)
+        # debiasing containment can lead to vals outside of 0-1 range!?
+        if max_containment >= 1:
+            return 1
+        elif max_containment <= 0:
+            return 0
+        else:
+            return max_containment
+
+    def max_containment_ani(self, other, *, downsample=False, max_containment=None, confidence=0.95, estimate_ci=False, prob_threshold=1e-3):  
+        "Use max_containment to estimate ANI between two MinHash objects."
+        if not (self.scaled and other.scaled):
+            raise TypeError("Error: can only calculate ANI for scaled MinHashes")
+        self_mh = self
+        other_mh = other
+        scaled = self.scaled
+        if downsample:
+            scaled = max(self_mh.scaled, other_mh.scaled)
+            self_mh = self.downsample(scaled=scaled)
+            other_mh = other.downsample(scaled=scaled)
+        max_containment = self_mh.max_containment_debiased(other_mh) #recalc debiased max containment
+        min_n_kmers = min(len(self_mh), len(other_mh))
+        n_kmers = min_n_kmers * scaled  # would be better if hll estimate - see #1798
+
+        c_aniresult = containment_to_distance(max_containment, self_mh.ksize, scaled,
+                                           n_unique_kmers=n_kmers,confidence=confidence,
+                                           estimate_ci = estimate_ci, prob_threshold=prob_threshold)
+        # null out ANI if either mh size estimation is inaccurate
+        if not self.size_is_accurate() or not other.size_is_accurate():
+            c_aniresult.size_is_inaccurate = True
+        return c_aniresult
+
+    def avg_containment(self, other, *, downsample=False):
+        """
+        Calculate average containment.
+        Note: this is average of the containments, *not* count_common/ avg_denom
+        """
+        if not (self.scaled and other.scaled):
+            raise TypeError("Error: can only calculate containment for scaled MinHashes")
+
+        c1 = self.contained_by(other, downsample)
+        c2 = other.contained_by(self, downsample)
+
+        return (c1 + c2)/2
+
+    def avg_containment_debiased(self, other, downsample=False):
+        """
+        Calculate average containment, debiased.
+        Note: this is average of the containments, *not* count_common/ avg_denom
+        """
+        if not (self.scaled and other.scaled):
+            raise TypeError("Error: can only calculate containment for scaled MinHashes")
+
+        c1 = self.contained_by_debiased(other, downsample)
+        c2 = other.contained_by_debiased(self, downsample)
+
+        return (c1 + c2)/2
+
+    def avg_containment_ani(self, other, *, downsample=False, prob_threshold=1e-3):
+        """
+        Calculate average containment ANI.
+        Note: this is average of the containment ANI's, *not* ANI using count_common/ avg_denom
+        """
+        if not (self.scaled and other.scaled):
+            raise TypeError("Error: can only calculate ANI for scaled MinHashes")
+        a1 = self.containment_ani(other, downsample=downsample, prob_threshold=prob_threshold).ani
+        a2 = other.containment_ani(self, downsample=downsample, prob_threshold=prob_threshold).ani
+        if any([a1 is None, a2 is None]):
+            return None
+        return (a1 + a2)/2
+
     def __add__(self, other):
         if not isinstance(other, MinHash):
             raise TypeError("can only add MinHash objects to MinHash objects!")
@@ -775,14 +977,23 @@ class MinHash(RustObject):
     def to_frozen(self):
         "Return a frozen copy of this MinHash that cannot be changed."
         new_mh = self.__copy__()
-        new_mh.__class__ = FrozenMinHash
+        new_mh.into_frozen()
         return new_mh
 
+    def into_frozen(self):
+        "Freeze this MinHash, preventing any changes."
+        self.__class__ = FrozenMinHash
+
     def inflate(self, from_mh):
-        "return a new MinHash object with abundances taken from 'from_mh'"
+        """return a new MinHash object with abundances taken from 'from_mh'
+
+        note that this implicitly does an intersection: hashes that have
+        no abundance in 'from_mh' are set to abundance 0 and removed from
+        'self'.
+        """
         if not self.track_abundance and from_mh.track_abundance:
             orig_abunds = from_mh.hashes
-            abunds = { h: orig_abunds[h] for h in self.hashes }
+            abunds = { h: orig_abunds.get(h, 0) for h in self.hashes }
 
             abund_mh = from_mh.copy_and_clear()
 
@@ -791,8 +1002,58 @@ class MinHash(RustObject):
 
             return abund_mh
         else:
-            raise ValueError("inflate operates on a flat MinHash and takes a MinHash object with track_abundance=True") 
-        
+            raise ValueError("inflate operates on a flat MinHash and takes a MinHash object with track_abundance=True")
+
+    @property
+    def sum_abundances(self):
+        if self.track_abundance:
+            return sum(v for v in self.hashes.values())
+        return None
+
+    @property
+    def mean_abundance(self):
+        if self.track_abundance:
+            return np.mean(list(self.hashes.values()))
+        return None
+
+    @property
+    def median_abundance(self):
+        if self.track_abundance:
+            return np.median(list(self.hashes.values()))
+        return None
+
+    @property
+    def std_abundance(self):
+        if self.track_abundance:
+            return np.std(list(self.hashes.values()))
+        return None
+
+    @property
+    def unique_dataset_hashes(self):
+        """
+        Approximate total number of hashes (num_hashes *scaled).
+        """
+        if not self.scaled:
+            raise TypeError("can only approximate unique_dataset_hashes for scaled MinHashes")
+        # TODO: replace set_size with HLL estimate when that gets implemented
+        return len(self) * self.scaled # + (self.ksize - 1) for bp estimation
+
+    def size_is_accurate(self, relative_error=0.20, confidence=0.95):
+        """
+        Computes the probability that the estimate: sketch_size * scaled deviates from the true
+        set_size by more than relative_error. This relies on the fact that the sketch_size
+        is binomially distributed with parameters sketch_size and 1/scaled. The two-sided Chernoff
+        bounds are used.
+        Returns True if probability is greater than or equal to the desired confidence.
+        """
+        if not self.scaled:
+            raise TypeError("Error: can only estimate dataset size for scaled MinHashes")
+        if any([not (0 <= relative_error <= 1), not (0 <= confidence <= 1)]):
+            raise ValueError("Error: relative error and confidence values must be between 0 and 1.")
+        # to do: replace unique_dataset_hashes with HLL estimation when it gets implemented 
+        probability = set_size_chernoff(self.unique_dataset_hashes, self.scaled, relative_error=relative_error)
+        return probability >= confidence
+
 
 class FrozenMinHash(MinHash):
     def add_sequence(self, *args, **kwargs):
@@ -828,12 +1089,16 @@ class FrozenMinHash(MinHash):
         if num and self.num == num:
             return self
 
-        return MinHash.downsample(self, num=num, scaled=scaled).to_frozen()
+        down_mh = MinHash.downsample(self, num=num, scaled=scaled)
+        down_mh.into_frozen()
+        return down_mh
 
     def flatten(self):
         if not self.track_abundance:
             return self
-        return MinHash.flatten(self).to_frozen()
+        flat_mh = MinHash.flatten(self)
+        flat_mh.into_frozen()
+        return flat_mh
 
     def __iadd__(self, *args, **kwargs):
         raise TypeError('FrozenMinHash does not support modification')
@@ -846,11 +1111,6 @@ class FrozenMinHash(MinHash):
         mut = MinHash.__new__(MinHash)
         state_tup = self.__getstate__()
 
-        # is protein/hp/dayhoff?
-        if state_tup[2] or state_tup[3] or state_tup[4]:
-            state_tup = list(state_tup)
-            # adjust ksize.
-            state_tup[1] = state_tup[1] * 3
         mut.__setstate__(state_tup)
         return mut
 
@@ -858,6 +1118,10 @@ class FrozenMinHash(MinHash):
         "Return a frozen copy of this MinHash that cannot be changed."
         return self
 
+    def into_frozen(self):
+        "Freeze this MinHash, preventing any changes."
+        pass
+
     def __setstate__(self, tup):
         "support pickling via __getstate__/__setstate__"
         (n, ksize, is_protein, dayhoff, hp, mins, _, track_abundance,
@@ -883,4 +1147,4 @@ class FrozenMinHash(MinHash):
 
     def __copy__(self):
         return self
-    copy = __copy__
\ No newline at end of file
+    copy = __copy__
