@@ -9,6 +9,11 @@ import zipfile
 from abc import ABC
 from pathlib import Path
 
+from ._lowlevel import ffi, lib
+from .utils import RustObject, rustcall, decode_str
+from .minhash import to_bytes
+
+
 class Storage(ABC):
 
     @abc.abstractmethod
@@ -89,7 +94,110 @@ class FSStorage(Storage):
         return path.read_bytes()
 
 
-class ZipStorage(Storage):
+class ZipStorage(RustObject, Storage):
+
+    __dealloc_func__ = lib.zipstorage_free
+
+    def __init__(self, path, *, mode="r"):
+        if mode == "w":
+            self.__inner = _RwZipStorage(path)
+        else:
+            self.__inner = None
+            path = os.path.abspath(path)
+            self._objptr = rustcall(lib.zipstorage_new, to_bytes(path), len(path))
+
+    @staticmethod
+    def can_open(location):
+        return zipfile.is_zipfile(location)
+
+    @property
+    def path(self):
+        if self.__inner:
+            return self.__inner.path
+        return decode_str(self._methodcall(lib.zipstorage_path))
+
+    @property
+    def subdir(self):
+        if self.__inner:
+            return self.__inner.subdir
+        return decode_str(self._methodcall(lib.zipstorage_subdir))
+
+    @subdir.setter
+    def subdir(self, value):
+        if self.__inner:
+            self.__inner.subdir = value
+        else:
+            self._methodcall(lib.zipstorage_set_subdir, to_bytes(value), len(value))
+
+    def _filenames(self):
+        if self.__inner:
+            return self.__inner._filenames()
+
+        size = ffi.new("uintptr_t *")
+        paths_ptr = self._methodcall(lib.zipstorage_filenames, size)
+        size = size[0]
+
+        paths = []
+        for i in range(size):
+            path = decode_str(paths_ptr[i][0])
+            paths.append(path)
+
+        return paths
+
+    def save(self, path, content, *, overwrite=False, compress=False):
+        if self.__inner:
+            return self.__inner.save(path, content, overwrite=overwrite, compress=compress)
+        raise NotImplementedError()
+
+    def load(self, path):
+        if self.__inner:
+            return self.__inner.load(path)
+
+        try:
+            size = ffi.new("uintptr_t *")
+            rawbuf = self._methodcall(lib.zipstorage_load, to_bytes(path), len(path), size)
+            size = size[0]
+
+            rawbuf = ffi.gc(rawbuf, lambda o: lib.nodegraph_buffer_free(o, size), size)
+            buf = ffi.buffer(rawbuf, size)
+
+            # TODO: maybe avoid the [:] here, it triggers a copy...
+            return buf[:]
+        except ValueError:
+            raise FileNotFoundError(path)
+
+    def list_sbts(self):
+        if self.__inner:
+            return self.__inner.list_sbts()
+
+        size = ffi.new("uintptr_t *")
+        paths_ptr = self._methodcall(lib.zipstorage_list_sbts, size)
+        size = size[0]
+
+        paths = []
+        for i in range(size):
+            path = decode_str(paths_ptr[i][0])
+            paths.append(path)
+
+        return paths
+
+    def init_args(self):
+        return {'path': self.path}
+
+    def flush(self):
+        if self.__inner:
+            self.__inner.flush()
+
+    def close(self):
+        if self.__inner:
+            self.__inner.close()
+
+    @staticmethod
+    def can_open(location):
+        return zipfile.is_zipfile(location)
+
+
+class _RwZipStorage(Storage):
 
     def __init__(self, path):
         self.path = os.path.abspath(path)
@@ -114,11 +222,14 @@ class ZipStorage(Storage):
             self.zipfile = zipfile.ZipFile(path, 'r')
             self.bufferzip = zipfile.ZipFile(BytesIO(), mode="w")
 
-        self.subdir = None
+        self.subdir = ""
         subdirs = [f for f in self.zipfile.namelist() if f.endswith("/")]
         if len(subdirs) == 1:
             self.subdir = subdirs[0]
 
+    def _filenames(self):
+        return [info.filename for info in self.zipfile.infolist()]
+
     def _content_matches(self, zf, path, content):
         info = zf.getinfo(path)
         entry_content = zf.read(info)
@@ -204,18 +315,22 @@ class ZipStorage(Storage):
         try:
             return self._load_from_zf(self.zipfile, path)
         except KeyError:
-            return self._load_from_zf(self.bufferzip, path)
-
-    def init_args(self):
-        return {'path': self.path}
+            if self.bufferzip:
+                return self._load_from_zf(self.bufferzip, path)
+            else:
+                raise FileNotFoundError(path)
 
     def close(self):
         # TODO: this is not ideal; checking for zipfile.fp is looking at
         # internal implementation details from CPython...
-        if self.zipfile is not None or self.bufferzip is not None:
-            self.flush(keep_closed=True)
-            self.zipfile.close()
-            self.zipfile = None
+
+        # might not have self.zipfile if was invalid zipfile and __init__
+        # failed.
+        if hasattr(self, 'zipfile'):
+            if self.zipfile is not None or self.bufferzip is not None:
+                self.flush(keep_closed=True)
+                self.zipfile.close()
+                self.zipfile = None
 
     def flush(self, *, keep_closed=False):
         # This is a bit complicated, but we have to deal with new data
@@ -270,7 +385,9 @@ class ZipStorage(Storage):
                     # Since there is no duplicated data, we can
                     # reopen self.zipfile in append mode and write the new data
                     self.zipfile.close()
-                    if not keep_closed:
+                    if keep_closed:
+                        raise Exception("unexpected error")
+                    else:
                         zf = zipfile.ZipFile(self.path, mode='a',
                                              compression=zipfile.ZIP_STORED)
                     for item in new_data:
@@ -280,10 +397,6 @@ class ZipStorage(Storage):
             self.bufferzip.close()
             self.bufferzip = None
 
-    @staticmethod
-    def can_open(location):
-        return zipfile.is_zipfile(location)
-
     def list_sbts(self):
         return [f for f in self.zipfile.namelist() if f.endswith(".sbt.json")]
 
