@@ -5,15 +5,19 @@ import sys
 import csv
 import json
 import os
-from collections import defaultdict
+from collections import defaultdict, namedtuple, Counter
+import json
+import re
 
 import screed
 import sourmash
 from sourmash.sourmash_args import FileOutput
 
-from sourmash.logging import set_quiet, error, notify, print_results, debug
+from sourmash.logging import (set_quiet, error, notify, print_results, debug,
+                              debug_literal)
 from sourmash import sourmash_args
 from sourmash.minhash import _get_max_hash_for_scaled
+from sourmash.manifest import CollectionManifest
 
 
 usage='''
@@ -36,6 +40,8 @@ subtract <signature> <other_sig> [...]    - subtract one or more signatures
 import [ ... ]                            - import a mash or other signature
 export <signature>                        - export a signature, e.g. to mash
 overlap <signature1> <signature2>         - see detailed comparison of sigs
+check <locations> --picklist ...          - check picklist against (many) sigs
+collect <locations> -o manifest.sqlmf     - collect sigs metadata into manifest
 
 ** Use '-h' to get subcommand-specific help, e.g.
 
@@ -48,12 +54,15 @@ def _check_abundance_compatibility(sig1, sig2):
         raise ValueError("incompatible signatures: track_abundance is {} in first sig, {} in second".format(sig1.minhash.track_abundance, sig2.minhash.track_abundance))
 
 
-def _extend_signatures_with_from_file(args):
+def _extend_signatures_with_from_file(args, *, target_attr='signatures'):
     # extend input signatures with --from-file
     if args.from_file:
         more_files = sourmash_args.load_pathlist_from_file(args.from_file)
-        args.signatures = list(args.signatures)
-        args.signatures.extend(more_files)
+
+        sigs = list(getattr(args, target_attr))
+        sigs.extend(more_files)
+        setattr(args, target_attr, sigs)
+
 
 def _set_num_scaled(mh, num, scaled):
     "set num and scaled values on a MinHash object"
@@ -77,6 +86,7 @@ def cat(args):
     set_quiet(args.quiet)
     moltype = sourmash_args.calculate_moltype(args)
     picklist = sourmash_args.load_picklist(args)
+    pattern_search = sourmash_args.load_include_exclude_db_patterns(args)
 
     encountered_md5sums = defaultdict(int)   # used by --unique
 
@@ -94,7 +104,8 @@ def cat(args):
                                                 picklist=picklist,
                                                 progress=progress,
                                                 yield_all_files=args.force,
-                                                force=args.force)
+                                                force=args.force,
+                                                pattern=pattern_search)
     for ss, sigloc in loader:
         md5 = ss.md5sum()
         encountered_md5sums[md5] += 1
@@ -103,17 +114,17 @@ def cat(args):
 
         save_sigs.add(ss)
 
-    notify('loaded {} signatures total.', len(save_sigs))
+    notify(f'loaded {len(save_sigs)} signatures total.')
     if picklist:
         sourmash_args.report_picklist(args, picklist)
 
     save_sigs.close()
 
-    notify('output {} signatures', len(save_sigs))
+    notify(f'output {len(save_sigs)} signatures')
 
     multiple_md5 = [ 1 for cnt in encountered_md5sums.values() if cnt > 1 ]
     if multiple_md5:
-        notify('encountered {} MinHashes multiple times', sum(multiple_md5))
+        notify(f'encountered {sum(multiple_md5)} MinHashes multiple times')
         if args.unique:
             notify('...and removed the duplicates, because --unique was specified.')
 
@@ -131,10 +142,10 @@ def split(args):
     output_scaled_template = '{md5sum}.k={ksize}.scaled={scaled}.{moltype}.dup={dup}.{basename}.sig'
     output_num_template = '{md5sum}.k={ksize}.num={num}.{moltype}.dup={dup}.{basename}.sig'
 
-    if args.outdir:
-        if not os.path.exists(args.outdir):
-            notify('Creating --outdir {}', args.outdir)
-            os.mkdir(args.outdir)
+    if args.output_dir:
+        if not os.path.exists(args.output_dir):
+            notify(f'Creating --output-dir {args.output_dir}')
+            os.mkdir(args.output_dir)
 
     progress = sourmash_args.SignatureLoadingProgress()
     loader = sourmash_args.load_many_signatures(args.signatures,
@@ -177,16 +188,16 @@ def split(args):
 
         output_names.add(output_name)
 
-        if args.outdir:
-            output_name = os.path.join(args.outdir, output_name)
+        if args.output_dir:
+            output_name = os.path.join(args.output_dir, output_name)
 
         if os.path.exists(output_name):
-            notify("** overwriting existing file {}".format(output_name))
+            notify(f"** overwriting existing file {format(output_name)}")
 
         # save!
         with open(output_name, 'wt') as outfp:
             sourmash.save_signatures([sig], outfp)
-            notify('writing sig to {}', output_name)
+            notify(f'writing sig to {output_name}')
 
     notify(f'loaded and split {len(progress)} signatures total.')
     if picklist:
@@ -197,21 +208,24 @@ def describe(args):
     """
     provide basic info on signatures
     """
-    set_quiet(args.quiet)
+    set_quiet(args.quiet, args.debug)
     moltype = sourmash_args.calculate_moltype(args)
     picklist = sourmash_args.load_picklist(args)
+    pattern_search = sourmash_args.load_include_exclude_db_patterns(args)
     _extend_signatures_with_from_file(args)
 
     # write CSV?
     w = None
-    csv_fp = None
+    csv_obj = None
     if args.csv:
-        # CTB: might want to switch to sourmash_args.FileOutputCSV here?
-        csv_fp = open(args.csv, 'w', newline='')
+        csv_obj = sourmash_args.FileOutputCSV(args.csv)
+        csv_fp = csv_obj.open()
+
         w = csv.DictWriter(csv_fp,
-                           ['signature_file', 'md5', 'ksize', 'moltype', 'num',
-                            'scaled', 'n_hashes', 'seed', 'with_abundance',
-                            'name', 'filename', 'license'],
+                           ['signature_file', 'md5', 'ksize', 'moltype',
+                            'num', 'scaled', 'n_hashes', 'seed',
+                            'with_abundance', 'name', 'filename', 'license',
+                            'sum_hashes'],
                            extrasaction='ignore')
         w.writeheader()
 
@@ -223,10 +237,12 @@ def describe(args):
                                                 picklist=picklist,
                                                 progress=progress,
                                                 yield_all_files=args.force,
-                                                force=args.force)
+                                                force=args.force,
+                                                pattern=pattern_search)
 
     for sig, location in loader:
         # extract info, write as appropriate.
+        signature_file = location
         mh = sig.minhash
         ksize = mh.ksize
         moltype = mh.moltype
@@ -234,6 +250,7 @@ def describe(args):
         num = mh.num
         seed = mh.seed
         n_hashes = len(mh)
+        sum_hashes = sum(mh.hashes.values())
         with_abundance = 0
         if mh.track_abundance:
             with_abundance = 1
@@ -255,11 +272,12 @@ source file: {p_filename}
 md5: {md5}
 k={ksize} molecule={moltype} num={num} scaled={scaled} seed={seed} track_abundance={with_abundance}
 size: {n_hashes}
+sum hashes: {sum_hashes}
 signature license: {license}
 ''', **locals())
 
-    if csv_fp:
-        csv_fp.close()
+    if csv_obj:
+        csv_obj.close()
 
     if picklist:
         sourmash_args.report_picklist(args, picklist)
@@ -269,43 +287,31 @@ def manifest(args):
     """
     build a signature manifest
     """
-    from sourmash.index import CollectionManifest
-
-    set_quiet(args.quiet)
-
-    # CTB: might want to switch to sourmash_args.FileOutputCSV here?
-    csv_fp = open(args.output, 'w', newline='')
-
-    CollectionManifest.write_csv_header(csv_fp)
-    w = csv.DictWriter(csv_fp, fieldnames=CollectionManifest.required_keys)
+    set_quiet(args.quiet, args.debug)
 
     try:
         loader = sourmash_args.load_file_as_index(args.location,
                                                   yield_all_files=args.force)
-    except Exception as exc:
-        error('\nError while reading signatures from {}:'.format(args.location))
-        error(str(exc))
-        error('(continuing)')
-        raise
-
-    n = 0
-    # Need to ignore existing manifests here! otherwise circularity...
-    try:
-        manifest_iter = loader._signatures_with_internal()
-    except NotImplementedError:
-        error("ERROR: manifests cannot be generated for this file.")
+    except ValueError as exc:
+        error(f"Cannot open '{args.location}' as a sourmash signature collection.")
+        error("Use -d/--debug for details.")
         sys.exit(-1)
 
-    for n, (sig, parent, loc) in enumerate(manifest_iter):
-        # extract info, write as appropriate.
-        row = CollectionManifest.make_manifest_row(sig, loc,
-                                                   include_signature=False)
-        w.writerow(row)
+    rebuild = True
+    if args.no_rebuild_manifest:
+        debug("sig manifest: not forcing rebuild.")
+        rebuild = False
+    else:
+        debug("sig manifest: forcing rebuild.")
 
-    notify(f'built manifest for {n} signatures total.')
+    manifest = sourmash_args.get_manifest(loader, require=True,
+                                          rebuild=rebuild)
 
-    if csv_fp:
-        csv_fp.close()
+    manifest.write_to_filename(args.output,
+                               database_format=args.manifest_format,
+                               ok_if_exists=args.force)
+    notify(f"manifest contains {len(manifest)} signatures total.")
+    notify(f"wrote manifest to '{args.output}' ({args.manifest_format})")
 
 
 def overlap(args):
@@ -321,8 +327,7 @@ def overlap(args):
     sig2 = sourmash.load_one_signature(args.signature2, ksize=args.ksize,
                                        select_moltype=moltype)
 
-    notify('loaded one signature each from {} and {}', args.signature1,
-           args.signature2)
+    notify(f'loaded one signature each from {args.signature1} and {args.signature2}')
 
     try:
         similarity = sig1.similarity(sig2)
@@ -450,7 +455,7 @@ def intersect(args):
     """
     intersect one or more signatures by taking the intersection of hashes.
 
-    This function always removes abundances.
+    This function always removes abundances unless -A specified.
     """
     set_quiet(args.quiet)
     moltype = sourmash_args.calculate_moltype(args)
@@ -475,42 +480,31 @@ def intersect(args):
             first_sig = sigobj
             mins = set(sigobj.minhash.hashes)
         else:
-            # check signature compatibility --
+            # check signature compatibility -- if no ksize/moltype specified
+            # 'first_sig' may be incompatible with later sigs.
             if not sigobj.minhash.is_compatible(first_sig.minhash):
                 error("incompatible minhashes; specify -k and/or molecule type.")
                 sys.exit(-1)
 
         mins.intersection_update(sigobj.minhash.hashes)
 
-    if len(progress) == 0:
-        error("no signatures to merge!?")
-        sys.exit(-1)
-
     # forcibly turn off track_abundance, unless --abundances-from set.
-    if not args.abundances_from:
-        intersect_mh = first_sig.minhash.copy_and_clear()
-        intersect_mh.track_abundance = False
-        intersect_mh.add_many(mins)
-        intersect_sigobj = sourmash.SourmashSignature(intersect_mh)
-    else:
-        notify('loading signature from {}, keeping abundances',
-               args.abundances_from)
+    intersect_mh = first_sig.minhash.copy_and_clear().flatten()
+    intersect_mh.add_many(mins)
+
+    # borrow abundances from a signature?
+    if args.abundances_from:
+        notify(f'loading signature from {args.abundances_from}, keeping abundances')
         abund_sig = sourmash.load_one_signature(args.abundances_from,
                                                 ksize=args.ksize,
                                                 select_moltype=moltype)
         if not abund_sig.minhash.track_abundance:
             error("--track-abundance not set on loaded signature?! exiting.")
             sys.exit(-1)
-        intersect_mh = abund_sig.minhash.copy_and_clear()
-        abund_mins = abund_sig.minhash.hashes
 
-        # do one last intersection
-        mins.intersection_update(abund_mins)
-        abund_mins = { k: abund_mins[k] for k in mins }
-
-        intersect_mh.set_abundances(abund_mins)
-        intersect_sigobj = sourmash.SourmashSignature(intersect_mh)
+        intersect_mh = intersect_mh.inflate(abund_sig.minhash)
 
+    intersect_sigobj = sourmash.SourmashSignature(intersect_mh)
     with FileOutput(args.output, 'wt') as fp:
         sourmash.save_signatures([intersect_sigobj], fp=fp)
 
@@ -519,6 +513,53 @@ def intersect(args):
         sourmash_args.report_picklist(args, picklist)
 
 
+def inflate(args):
+    """
+    inflate one or more other signatures from the first.
+    """
+    set_quiet(args.quiet)
+    moltype = sourmash_args.calculate_moltype(args)
+    picklist = sourmash_args.load_picklist(args)
+
+    inflate_sig = sourmash_args.load_query_signature(args.signature_from,
+                                                     ksize=args.ksize,
+                                                     select_moltype=moltype)
+    inflate_from_mh = inflate_sig.minhash
+    ksize = inflate_from_mh.ksize
+    moltype = inflate_from_mh.moltype
+
+    if not inflate_from_mh.track_abundance:
+        error(f"ERROR: signature '{inflate_sig.name}' from ")
+        error(f"file '{args.signature_from}' has no abundances.")
+        sys.exit(-1)
+
+    # start loading!
+    progress = sourmash_args.SignatureLoadingProgress()
+    loader = sourmash_args.load_many_signatures(args.other_sigs,
+                                                ksize=ksize,
+                                                moltype=moltype,
+                                                picklist=picklist,
+                                                progress=progress,
+                                                yield_all_files=args.force,
+                                                force=args.force)
+
+    with sourmash_args.SaveSignaturesToLocation(args.output) as save_sigs:
+        for sigobj, sigloc in loader:
+            inflated_mh = sigobj.minhash.inflate(inflate_from_mh)
+            inflated_sigobj = sourmash.SourmashSignature(inflated_mh,
+                                                         name=sigobj.name)
+
+            save_sigs.add(inflated_sigobj)
+
+    if len(progress) == 0:
+        error("no signatures to inflate!?")
+        sys.exit(-1)
+
+    notify(f'loaded and intersected {len(save_sigs)} signatures')
+    if picklist:
+        sourmash_args.report_picklist(args, picklist)
+
+
 def subtract(args):
     """
     subtract one or more signatures from another
@@ -529,6 +570,9 @@ def subtract(args):
     from_sigfile = args.signature_from
     from_sigobj = sourmash.load_one_signature(from_sigfile, ksize=args.ksize, select_moltype=moltype)
 
+    if args.abundances_from:    # it's ok to work with abund signatures if -A.
+        args.flatten = True
+
     from_mh = from_sigobj.minhash
     if from_mh.track_abundance and not args.flatten:
         error('Cannot use subtract on signatures with abundance tracking, sorry!')
@@ -536,7 +580,7 @@ def subtract(args):
 
     subtract_mins = set(from_mh.hashes)
 
-    notify('loaded signature from {}...', from_sigfile, end='\r')
+    notify(f'loaded signature from {from_sigfile}...', end='\r')
 
     progress = sourmash_args.SignatureLoadingProgress()
 
@@ -555,15 +599,28 @@ def subtract(args):
 
             subtract_mins -= set(sigobj.minhash.hashes)
 
-            notify('loaded and subtracted signatures from {}...', sigfile, end='\r')
+            notify(f'loaded and subtracted signatures from {sigfile}...', end='\r')
 
     if not len(progress):
         error("no signatures to subtract!?")
         sys.exit(-1)
 
-    subtract_mh = from_sigobj.minhash.copy_and_clear()
+    # build new minhash with new mins
+    subtract_mh = from_sigobj.minhash.copy_and_clear().flatten()
     subtract_mh.add_many(subtract_mins)
 
+    # borrow abundances from somewhere?
+    if args.abundances_from:
+        notify(f'loading signature from {args.abundances_from}, keeping abundances')
+        abund_sig = sourmash.load_one_signature(args.abundances_from,
+                                                ksize=args.ksize,
+                                                select_moltype=moltype)
+        if not abund_sig.minhash.track_abundance:
+            error("--track-abundance not set on loaded signature?! exiting.")
+            sys.exit(-1)
+
+        subtract_mh = subtract_mh.inflate(abund_sig.minhash)
+
     subtract_sigobj = sourmash.SourmashSignature(subtract_mh)
 
     with FileOutput(args.output, 'wt') as fp:
@@ -579,9 +636,9 @@ def rename(args):
     set_quiet(args.quiet, args.quiet)
     moltype = sourmash_args.calculate_moltype(args)
     picklist = sourmash_args.load_picklist(args)
+    pattern_search = sourmash_args.load_include_exclude_db_patterns(args)
     _extend_signatures_with_from_file(args)
 
-
     save_sigs = sourmash_args.SaveSignaturesToLocation(args.output)
     save_sigs.open()
 
@@ -593,9 +650,11 @@ def rename(args):
                                                 picklist=picklist,
                                                 progress=progress,
                                                 yield_all_files=args.force,
-                                                force=args.force)
+                                                force=args.force,
+                                                pattern=pattern_search)
 
     for sigobj, sigloc in loader:
+        sigobj = sigobj.to_mutable()
         sigobj._name = args.name
         save_sigs.add(sigobj)
 
@@ -613,49 +672,70 @@ def extract(args):
     set_quiet(args.quiet)
     moltype = sourmash_args.calculate_moltype(args)
     picklist = sourmash_args.load_picklist(args)
+    pattern_search = sourmash_args.load_include_exclude_db_patterns(args)
     _extend_signatures_with_from_file(args)
 
     # further filtering on md5 or name?
+    filter_fn = None
     if args.md5 is not None or args.name is not None:
-        def filter_fn(ss):
+        def filter_fn(row):
             # match?
             keep = False
-            if args.name and args.name in str(ss):
-                keep = True
-            if args.md5 and args.md5 in ss.md5sum():
+            if args.name:
+                name = row['name'] or row['filename']
+                if args.name in name:
+                    keep = True
+            if args.md5 and args.md5 in row['md5']:
                 keep = True
 
             return keep
-    else:
-        # whatever comes out of the database is fine
-        filter_fn = lambda x: True
 
     # ok! filtering defined, let's go forward
     save_sigs = sourmash_args.SaveSignaturesToLocation(args.output)
     save_sigs.open()
 
     # start loading!
-    progress = sourmash_args.SignatureLoadingProgress()
-    loader = sourmash_args.load_many_signatures(args.signatures,
-                                                ksize=args.ksize,
-                                                moltype=moltype,
-                                                picklist=picklist,
-                                                progress=progress,
-                                                yield_all_files=args.force,
-                                                force=args.force)
-    for ss, sigloc in loader:
-        if filter_fn(ss):
+    total_rows_examined = 0
+    for filename in args.signatures:
+        idx = sourmash_args.load_file_as_index(filename,
+                                               yield_all_files=args.force)
+
+        idx = idx.select(ksize=args.ksize, moltype=moltype)
+
+        idx = sourmash_args.apply_picklist_and_pattern(idx, picklist,
+                                                       pattern_search)
+
+        manifest = sourmash_args.get_manifest(idx)
+        total_rows_examined += len(manifest)
+
+        # do the extra pattern matching on name/md5 that is part of 'extract'.
+        # CTB: This should be deprecated and removed at some point, since
+        # --include/--exclude now do the same thing.
+        if filter_fn and not pattern_search:
+            sub_manifest = manifest.filter_rows(filter_fn)
+            sub_picklist = sub_manifest.to_picklist()
+
+            try:
+                idx = idx.select(picklist=sub_picklist)
+            except ValueError:
+                error("** This input collection doesn't support 'extract' with picklists or patterns.")
+                error("** EXITING.")
+                error("**")
+                error("** You can use 'sourmash sig cat' with a picklist or pattern,")
+                error("** and then pipe the output to 'sourmash sig extract")
+                sys.exit(-1)
+
+        for ss in idx.signatures():
             save_sigs.add(ss)
 
-    notify(f"loaded {len(progress)} total that matched ksize & molecule type")
+    notify(f"loaded {total_rows_examined} total that matched ksize & molecule type")
     if not save_sigs:
         error("no matching signatures to save!")
         sys.exit(-1)
 
     save_sigs.close()
 
-    notify("extracted {} signatures from {} file(s)", len(save_sigs),
-           len(args.signatures))
+    notify(f"extracted {len(save_sigs)} signatures from {len(args.signatures)} file(s)")
 
     if picklist:
         sourmash_args.report_picklist(args, picklist)
@@ -689,8 +769,7 @@ def filter(args):
         for ss in siglist:
             mh = ss.minhash
             if not mh.track_abundance:
-                notify('ignoring signature {} - track_abundance not set.',
-                       ss)
+                notify(f'ignoring signature {ss} - track_abundance not set.')
                 continue
 
             abunds = mh.hashes
@@ -704,6 +783,7 @@ def filter(args):
             filtered_mh = mh.copy_and_clear()
             filtered_mh.set_abundances(abunds2)
 
+            ss = ss.to_mutable()
             ss.minhash = filtered_mh
 
             save_sigs.add(ss)
@@ -711,8 +791,7 @@ def filter(args):
     save_sigs.close()
 
     notify(f"loaded {len(progress)} total that matched ksize & molecule type")
-    notify("extracted {} signatures from {} file(s)", len(save_sigs),
-           len(args.signatures))
+    notify(f"extracted {len(save_sigs)} signatures from {len(args.signatures)} file(s)")
 
 
 def flatten(args):
@@ -746,14 +825,14 @@ def flatten(args):
             if args.name not in ss.name:
                 continue        # skip
 
+        ss = ss.to_mutable()
         ss.minhash = ss.minhash.flatten()
         save_sigs.add(ss)
 
     save_sigs.close()
 
     notify(f"loaded {len(progress)} total that matched ksize & molecule type")
-    notify("extracted {} signatures from {} file(s)", len(save_sigs),
-           len(args.signatures))
+    notify(f"extracted {len(save_sigs)} signatures from {len(args.signatures)} file(s)")
     if picklist:
         sourmash_args.report_picklist(args, picklist)
 
@@ -789,7 +868,8 @@ def downsample(args):
                                                 yield_all_files=args.force,
                                                 force=args.force)
     for ss, sigloc in loader:
-        mh = ss.minhash
+        sigobj = ss.to_mutable()
+        mh = sigobj.minhash
 
         if args.scaled:
             # downsample scaled to scaled? straightforward.
@@ -818,10 +898,9 @@ def downsample(args):
                 mh_new = mh.copy()
                 _set_num_scaled(mh_new, args.num_hashes, 0)
 
-        ss.minhash = mh_new
 
-        # save!
-        save_sigs.add(ss)
+        sigobj.minhash = mh_new
+        save_sigs.add(sigobj)
 
     save_sigs.close()
 
@@ -861,7 +940,7 @@ def sig_import(args):
                     e.add_many(hashes)
                     s = sourmash.SourmashSignature(e, filename=name)
                     siglist.append(s)
-                    notify('loaded signature: {} {}', name, s.md5sum()[:8])
+                    notify(f'loaded signature: {name} {s.md5sum()[:8]}')
     else:
         for filename in args.filenames:
             with open(filename) as fp:
@@ -883,7 +962,7 @@ def sig_import(args):
             s = sourmash.SourmashSignature(mh, filename=filename)
             siglist.append(s)
 
-    notify('saving {} signatures to JSON', len(siglist))
+    notify(f'saving {len(siglist)} signatures to JSON')
     with FileOutput(args.output, 'wt') as fp:
         sourmash.save_signatures(siglist, fp)
 
@@ -914,7 +993,7 @@ def export(args):
 
     with FileOutput(args.output, 'wt') as fp:
         print(json.dumps(x), file=fp)
-    notify("exported signature {} ({})", query, query.md5sum()[:8])
+    notify(f"exported signature {query} ({query.md5sum()[:8]})")
 
 
 def kmers(args):
@@ -1121,6 +1200,297 @@ def kmers(args):
         notify("NOTE: see --save-kmers or --save-sequences for output options.")
 
 
+_SketchInfo = namedtuple('_SketchInfo', 'ksize, moltype, scaled, num, abund')
+
+
+def _summarize_manifest(manifest):
+    info_d = {}
+
+    # use a namedtuple to track counts of distinct sketch types and n hashes
+    total_size = 0
+    counter = Counter()
+    hashcounts = Counter()
+    for row in manifest.rows:
+        ski = _SketchInfo(ksize=row['ksize'], moltype=row['moltype'],
+                          scaled=row['scaled'], num=row['num'],
+                          abund=row['with_abundance'])
+        counter[ski] += 1
+        hashcounts[ski] += row['n_hashes']
+        total_size += row['n_hashes']
+
+    # store in info_d
+    info_d['total_hashes'] = total_size
+    sketch_info = []
+    for ski, count in counter.items():
+        sketch_d = dict(ski._asdict())
+        sketch_d['count'] = count
+        sketch_d['n_hashes'] = hashcounts[ski]
+        sketch_info.append(sketch_d)
+    info_d['sketch_info'] = sketch_info
+
+    return info_d
+
+
+# NOTE: also aliased as 'summarize'
+def fileinfo(args):
+    """
+    provide summary information on the given path (collection, index, etc.)
+    """
+    set_quiet(args.quiet, args.debug)
+
+    text_out = False
+    if not args.json_out:
+        text_out = True
+
+    # load as index!
+    try:
+        notify(f"** loading from '{args.path}'")
+        idx = sourmash_args.load_file_as_index(args.path,
+                                               yield_all_files=args.force)
+    except ValueError:
+        error(f"Cannot open '{args.path}' as a sourmash signature collection.")
+        error("Use -d/--debug for details.")
+        sys.exit(-1)
+
+    print_bool = lambda x: "yes" if x else "no"
+    print_none = lambda x: "n/a" if x is None else x
+
+    info_d = {}
+    info_d['path_filetype'] = type(idx).__name__
+    info_d['location'] = "" if not idx.location else idx.location
+    info_d['is_database'] = bool(idx.is_database)
+    info_d['has_manifest'] = bool(idx.manifest)
+    info_d['num_sketches'] = len(idx)
+
+    if text_out:
+        print_results(f"path filetype: {info_d['path_filetype']}")
+        print_results(f"location: {info_d['location']}")
+        print_results(f"is database? {print_bool(info_d['is_database'])}")
+        print_results(f"has manifest? {print_bool(info_d['has_manifest'])}")
+        print_results(f"num signatures: {info_d['num_sketches']}")
+
+    # also have arg to fileinfo to force recalculation
+    notify("** examining manifest...")
+
+    manifest = sourmash_args.get_manifest(idx, rebuild=args.rebuild_manifest,
+                                          require=False)
+
+    if manifest is None:
+        # actually can't find any file type to trigger this, but leaving it
+        # in for future eventualities, I guess?
+        notify("** no manifest and cannot be generated; exiting.")
+        sys.exit(0)
+
+    info_d.update(_summarize_manifest(manifest))
+
+    if text_out:
+        print_results(f"total hashes: {info_d['total_hashes']}")
+        print_results("summary of sketches:")
+
+        for ski in info_d['sketch_info']:
+            mh_type = f"num={ski['num']}" if ski['num'] else f"scaled={ski['scaled']}"
+            mh_abund = ", abund" if ski['abund'] else ""
+
+            sketch_str = f"{ski['count']} sketches with {ski['moltype']}, k={ski['ksize']}, {mh_type}{mh_abund}"
+
+            print_results(f"   {sketch_str: <50} {ski['n_hashes']} total hashes")
+
+    else:
+        assert args.json_out
+        print(json.dumps(info_d))
+
+
+def check(args):
+    """
+    check signature db(s) against a picklist.
+    """
+    from sourmash.picklist import PickStyle
+    set_quiet(args.quiet, args.debug)
+    moltype = sourmash_args.calculate_moltype(args)
+    picklist = sourmash_args.load_picklist(args)
+    pattern_search = sourmash_args.load_include_exclude_db_patterns(args)
+    _extend_signatures_with_from_file(args)
+
+    if not picklist:
+        error("** No picklist provided?! Exiting.")
+        sys.exit(-1)
+
+    if picklist.pickstyle == PickStyle.EXCLUDE and args.output_missing:
+        error("** ERROR: Cannot use an 'exclude' picklist with '-o/--output-missing'")
+        sys.exit(-1)
+
+    # require manifests?
+    require_manifest = True
+    if args.no_require_manifest:
+        require_manifest = False
+        debug("sig check: manifest will not be required")
+    else:
+        debug("sig check: manifest required")
+
+    total_manifest_rows = CollectionManifest([])
+
+    # start loading!
+    total_rows_examined = 0
+    for filename in args.signatures:
+        idx = sourmash_args.load_file_as_index(filename,
+                                               yield_all_files=args.force)
+
+        idx = idx.select(ksize=args.ksize, moltype=moltype)
+
+        if idx.manifest is None and require_manifest:
+            error(f"ERROR on filename '{filename}'.")
+            error("sig check requires a manifest by default, but no manifest present.")
+            error("specify --no-require-manifest to dynamically generate one.")
+            sys.exit(-1)
+
+        # has manifest, or ok to build (require_manifest=False) - continue!
+        new_manifest = sourmash_args.get_manifest(idx, require=True)
+        sub_manifest = new_manifest.select_to_manifest(picklist=picklist)
+        total_rows_examined += len(new_manifest)
+
+        # rewrite locations so that each signature can be found by filename
+        # of its container; this follows `sig collect` logic.
+        rows = []
+        for row in sub_manifest.rows:
+            row['internal_location'] = filename
+            total_manifest_rows.add_row(row)
+
+        debug_literal(f"examined {len(new_manifest)} new rows, found {len(sub_manifest)} matching rows")
+
+    notify(f"loaded {total_rows_examined} signatures.")
+
+    sourmash_args.report_picklist(args, picklist)
+
+    # output picklist of non-matching in same format as input picklist
+    n_missing = len(picklist.pickset - picklist.found)
+    if args.output_missing and n_missing:
+        pickfile = picklist.pickfile
+
+        # go through the input file and pick out missing rows.
+        n_input = 0
+        n_output = 0
+
+        with sourmash_args.FileInputCSV(pickfile) as r:
+            with open(args.output_missing, "w", newline='') as outfp:
+                w = csv.DictWriter(outfp, fieldnames=r.fieldnames)
+                w.writeheader()
+
+                for row in r:
+                    n_input += 1
+                    if not picklist.matched_csv_row(row):
+                        n_output += 1
+                        w.writerow(row)
+        notify(f"saved {n_output} non-matching rows of {n_input} picklist rows to '{args.output_missing}'")
+    elif args.output_missing:
+        notify(f"(no remaining picklist entries; not saving to '{args.output_missing}')")
+
+    # save manifest of matching!
+    if args.save_manifest_matching and total_manifest_rows:
+        mf = total_manifest_rows
+        mf.write_to_filename(args.save_manifest_matching,
+                             database_format=args.manifest_format)
+        notify(f"wrote {len(mf)} matching manifest rows to '{args.save_manifest_matching}'")
+    elif args.save_manifest_matching:
+        notify(f"(not saving matching manifest to '{args.save_manifest_matching}' because no matches)")
+
+    if args.fail_if_missing and n_missing:
+        error("** ERROR: missing values, and --fail-if-missing requested. Exiting.")
+        sys.exit(-1)
+
+
+def collect(args):
+    "Collect signature metadata across many locations, save to manifest"
+    # TODO:
+    # test what happens with directories :)
+    set_quiet(False, args.debug)
+
+    if os.path.exists(args.output):
+        if args.merge_previous:
+            pass
+        else:
+            error(f"ERROR: '{args.output}' already exists!")
+            error(f"ERROR: please remove it, or use --merge-previous to merge")
+            sys.exit(-1)
+    elif args.merge_previous:
+        notify(f"WARNING: --merge-previous specified, but output file '{args.output}' does not already exist?")
+
+    # load previous manifest for --merge-previous. This gets tricky with
+    # mismatched manifest types, which we forbid.
+    try:
+        if args.manifest_format == 'sql':
+            # create on-disk manifest
+            from sourmash.index.sqlite_index import SqliteCollectionManifest
+
+            if args.merge_previous:
+                collected_mf = SqliteCollectionManifest.create_or_open(args.output)
+            else:
+                collected_mf = SqliteCollectionManifest.create(args.output)
+        else:
+            # create in-memory manifest that will be saved as CSV
+            assert args.manifest_format == 'csv'
+
+            if args.merge_previous and os.path.exists(args.output):
+                collected_mf = CollectionManifest.load_from_filename(args.output)
+            else:
+                collected_mf = CollectionManifest()
+
+            if not isinstance(collected_mf, CollectionManifest):
+                raise Exception
+    except:
+        error(f"ERROR loading '{args.output}' with --merge-previous. Is it of type {args.manifest_format}?")
+        sys.exit(-1)
+
+    if args.merge_previous:
+        notify(f"merging new locations with {len(collected_mf)} previous rows.")
+
+    # require manifests? yes by default, since generating can be slow.
+    require_manifest = True
+    if args.no_require_manifest:
+        require_manifest = False
+        debug("sig check: manifest will not be required")
+    else:
+        debug("sig check: manifest required")
+
+    n_files = 0
+
+    # load from_file
+    _extend_signatures_with_from_file(args, target_attr='locations')
+
+    # convert to abspath
+    if args.abspath:
+        args.locations = [ os.path.abspath(iloc) for iloc in args.locations ]
+
+    # iterate through, loading all the manifests from all the locations.
+    for n_files, loc in enumerate(args.locations):
+        notify(f"Loading signature information from {loc}.")
+
+        if n_files % 100 == 0:
+            notify(f'... loaded {len(collected_mf)} sigs from {n_files} files')
+        idx = sourmash.load_file_as_index(loc)
+        if idx.manifest is None and require_manifest:
+            error(f"ERROR on location '{loc}'")
+            error(f"sig collect requires a manifest by default, but no manifest present.")
+            error("specify --no-require-manifest to dynamically generate one.")
+            sys.exit(-1)
+
+        mf = sourmash_args.get_manifest(idx)
+
+        rows = []
+        for row in mf.rows:
+            row['internal_location'] = loc
+            collected_mf.add_row(row)
+
+    if args.manifest_format == 'csv':
+        collected_mf.write_to_filename(args.output, database_format='csv',
+                                       ok_if_exists=args.merge_previous)
+    else:
+        collected_mf.close()
+
+    notify(f"saved {len(collected_mf)} manifest rows to '{args.output}'")
+
+    return 0
+
+
 def main(arglist=None):
     args = sourmash.cli.get_parser().parse_args(arglist)
     submod = getattr(sourmash.cli.sig, args.subcmd)
