@@ -6,6 +6,7 @@ import sys
 import os
 import weakref
 from enum import Enum
+import contextlib
 
 from .logging import error
 from . import MinHash
@@ -142,13 +143,45 @@ class SourmashSignature(RustObject):
         return self.minhash.similarity(other.minhash, ignore_abundance=True,
                                        downsample=False)
 
+    def jaccard_ani(self, other, *, downsample=False, jaccard=None, prob_threshold=1e-3, err_threshold=1e-4):
+        "Use jaccard to estimate ANI between two FracMinHash signatures."
+        return self.minhash.jaccard_ani(other.minhash, downsample=downsample,
+                                        jaccard=jaccard, prob_threshold=prob_threshold,
+                                        err_threshold=err_threshold)
+
     def contained_by(self, other, downsample=False):
         "Compute containment by the other signature. Note: ignores abundance."
-        return self.minhash.contained_by(other.minhash, downsample)
+        return self.minhash.contained_by(other.minhash, downsample=downsample)
+
+    def containment_ani(self, other, *, downsample=False, containment=None, confidence=0.95, estimate_ci=False):
+        "Use containment to estimate ANI between two FracMinHash signatures."
+        return self.minhash.containment_ani(other.minhash, downsample=downsample,
+                                        containment=containment, confidence=confidence,
+                                        estimate_ci=estimate_ci)
 
     def max_containment(self, other, downsample=False):
         "Compute max containment w/other signature. Note: ignores abundance."
-        return self.minhash.max_containment(other.minhash, downsample)
+        return self.minhash.max_containment(other.minhash, downsample=downsample)
+
+    def max_containment_ani(self, other, *, downsample=False, max_containment=None, confidence=0.95, estimate_ci=False):
+        "Use max containment to estimate ANI between two FracMinHash signatures."
+        return self.minhash.max_containment_ani(other.minhash, downsample=downsample,
+                                                max_containment=max_containment, confidence=confidence,
+                                                estimate_ci=estimate_ci)
+
+    def avg_containment(self, other, downsample=False):
+        """
+        Calculate average containment.
+        Note: this is average of the containments, *not* count_common/ avg_denom
+        """
+        return self.minhash.avg_containment(other.minhash, downsample=downsample)
+
+    def avg_containment_ani(self, other, *, downsample=False):
+        """
+        Calculate average containment ANI.
+        Note: this is average of the containment ANI's, *not* ANI using count_common/ avg_denom
+        """
+        return self.minhash.avg_containment_ani(other.minhash, downsample=downsample)
 
     def add_sequence(self, sequence, force=False):
         self._methodcall(lib.signature_add_sequence, to_bytes(sequence), force)
@@ -193,10 +226,8 @@ class SourmashSignature(RustObject):
         )
 
     def __copy__(self):
-        mh = self.minhash
-        mh = mh.to_frozen()
         a = SourmashSignature(
-            mh,
+            self.minhash,
             name=self.name,
             filename=self.filename,
         )
@@ -204,6 +235,86 @@ class SourmashSignature(RustObject):
 
     copy = __copy__
 
+    def to_frozen(self):
+        "Return a frozen copy of this signature."
+        new_ss = self.copy()
+        new_ss.__class__ = FrozenSourmashSignature
+        return new_ss
+
+    def to_mutable(self):
+        "Return a mutable copy of this signature."
+        return self.copy()
+
+    def into_frozen(self):
+        "Freeze this signature, preventing attribute changes."
+        # this will always be the case b/c minhash property returns FrozenMH:
+        # assert isinstance(self.minhash, FrozenMinHash)
+        self.__class__ = FrozenSourmashSignature
+
+
+class FrozenSourmashSignature(SourmashSignature):
+    "Frozen (immutable) signature class."
+
+    @SourmashSignature.minhash.setter
+    def minhash(self, value):
+        raise ValueError("cannot set .minhash on FrozenSourmashSignature")
+
+    @SourmashSignature._name.setter
+    def _name(self, value):
+        raise ValueError("cannot set ._name on FrozenSourmashSignature")
+
+    @SourmashSignature.name.setter
+    def name(self, value):
+        raise ValueError("cannot set .name on FrozenSourmashSignature")
+
+    @SourmashSignature.filename.setter
+    def filename(self, value):
+        raise ValueError("cannot set .filename on FrozenSourmashSignature")
+
+    def add_sequence(self, sequence, force=False):
+        raise ValueError("cannot add sequence data to FrozenSourmashSignature")
+
+    def add_protein(self, sequence):
+        raise ValueError("cannot add protein sequence to FrozenSourmashSignature")
+
+    def __copy__(self):
+        return self
+    copy = __copy__
+
+    def to_frozen(self):
+        "Return a frozen copy of this signature."
+        return self
+
+    def to_mutable(self):
+        "Turn this object into a mutable object."
+        mut = SourmashSignature.__new__(SourmashSignature)
+        state_tup = self.__getstate__()
+        mut.__setstate__(state_tup)
+        return mut
+
+    def into_frozen(self):
+        "Freeze this signature, preventing attribute changes."
+        self.__class__ = FrozenSourmashSignature
+
+    @contextlib.contextmanager
+    def update(self):
+        """Make a mutable copy of this signature for modification, then freeze.
+
+        This is a context manager that implements:
+
+        new_sig = this_sig.copy()
+        new_sig.to_mutable()
+        # modify new_sig
+        new_sig.into_frozen()
+
+        This could be made more efficient by _not_ copying the signature,
+        but that is non-intuitive and leads to hard-to-find bugs.
+        """
+        new_copy = self.to_mutable()
+        yield new_copy
+        new_copy.into_frozen()
+
+
 def _detect_input_type(data):
     """\
     Determine how to load input from `data`. Returns SigInput enum.
@@ -240,7 +351,7 @@ def load_signatures(
 ):
     """Load a JSON string with signatures into classes.
 
-    Returns list of SourmashSignature objects.
+    Returns iterator over SourmashSignature objects.
 
     Note, the order is not necessarily the same as what is in the source file.
     """
@@ -310,7 +421,7 @@ def load_signatures(
             sigs.append(sig)
 
         for sig in sigs:
-            yield sig
+            yield sig.to_frozen()
 
     except Exception as e:
         if do_raise:
