@@ -1,13 +1,50 @@
 """
 Utility functions for sourmash CLI commands.
+
+The sourmash_args submodule contains functions that help with various
+command-line functions. Library functions in this module often directly
+send output to stdout/stderr in support of the CLI, and/or call
+sys.exit to exit.
+
+argparse functionality:
+
+* check_scaled_bounds(args) -- check that --scaled is reasonable
+* check_num_bounds(args) -- check that --num is reasonable
+* get_moltype(args) -- verify that moltype selected is legit
+* calculate_moltype(args) -- confirm that only one moltype was selected
+* load_picklist(args) -- create a SignaturePicklist from --picklist args
+* report_picklist(args, picklist) -- report on picklist value usage/matches
+* load_include_exclude_db_patterns(args) -- load --include-db-pattern / --exclude-db-pattern
+* apply_picklist_and_pattern(db, ...) -- subselect db on picklist and pattern
+
+signature/database loading functionality:
+
+* load_query_signature(filename, ...) -- load a single signature for query
+* traverse_find_sigs(filenames, ...) -- find all .sig and .sig.gz files
+* load_dbs_and_sigs(filenames, query, ...) -- load databases & signatures
+* load_file_as_index(filename, ...) -- load a sourmash.Index class
+* load_file_as_signatures(filename, ...) -- load a list of signatures
+* load_pathlist_from_file(filename) -- load a list of paths from a file
+* load_many_signatures(locations) -- load many signatures from many files
+* get_manifest(idx) -- retrieve or build a manifest from an Index
+* class SignatureLoadingProgress - signature loading progress bar
+
+signature and file output functionality:
+
+* SaveSignaturesToLocation(filename) - bulk signature output
+* class FileOutput - file output context manager that deals w/stdout well
+* class FileOutputCSV - file output context manager for CSV files
 """
 import sys
 import os
+import csv
 from enum import Enum
 import traceback
 import gzip
+from io import StringIO, TextIOWrapper
+import re
 import zipfile
-from io import StringIO
+import contextlib
 
 import screed
 import sourmash
@@ -19,14 +56,40 @@ import sourmash.exceptions
 from .logging import notify, error, debug_literal
 
 from .index import (LinearIndex, ZipFileLinearIndex, MultiIndex)
+from .index.sqlite_index import load_sqlite_index, SqliteIndex
 from . import signature as sigmod
 from .picklist import SignaturePicklist, PickStyle
 from .manifest import CollectionManifest
+import argparse
 
 
 DEFAULT_LOAD_K = 31
 
 
+def check_scaled_bounds(arg):
+    f = float(arg)
+
+    if f < 0:
+        raise argparse.ArgumentTypeError(f"ERROR: scaled value must be positive")
+    if f < 100:
+        notify('WARNING: scaled value should be >= 100. Continuing anyway.')
+    if f > 1e6:
+        notify('WARNING: scaled value should be <= 1e6. Continuing anyway.')
+    return f
+
+
+def check_num_bounds(arg):
+    f = int(arg)
+
+    if f < 0:
+        raise argparse.ArgumentTypeError(f"ERROR: num value must be positive")
+    if f < 50:
+        notify('WARNING: num value should be >= 50. Continuing anyway.')
+    if f > 50000:
+        notify('WARNING: num value should be <= 50000. Continuing anyway.')
+    return f
+
+
 def get_moltype(sig, require=False):
     mh = sig.minhash
     if mh.moltype in ('DNA', 'dayhoff', 'hp', 'protein'):
@@ -67,15 +130,15 @@ def load_picklist(args):
     if args.picklist:
         try:
             picklist = SignaturePicklist.from_picklist_args(args.picklist)
+
+            notify(f"picking column '{picklist.column_name}' of type '{picklist.coltype}' from '{picklist.pickfile}'")
+
+            n_empty_val, dup_vals = picklist.load(picklist.pickfile, picklist.column_name)
         except ValueError as exc:
             error("ERROR: could not load picklist.")
             error(str(exc))
             sys.exit(-1)
 
-        notify(f"picking column '{picklist.column_name}' of type '{picklist.coltype}' from '{picklist.pickfile}'")
-
-        n_empty_val, dup_vals = picklist.load(picklist.pickfile, picklist.column_name)
-
         notify(f"loaded {len(picklist.pickset)} distinct values into picklist.")
         if n_empty_val:
             notify(f"WARNING: {n_empty_val} empty values in column '{picklist.column_name}' in picklist file")
@@ -100,6 +163,46 @@ def report_picklist(args, picklist):
             sys.exit(-1)
 
 
+def load_include_exclude_db_patterns(args):
+    if args.picklist and (args.include_db_pattern or args.exclude_db_pattern):
+        error("ERROR: --picklist and --include-db-pattern/--exclude cannot be used together.")
+        sys.exit(-1)
+
+    if args.include_db_pattern and args.exclude_db_pattern:
+        error("ERROR: --include-db-pattern and --exclude-db-pattern cannot be used together.")
+        sys.exit(-1)
+
+    if args.include_db_pattern:
+        pattern = re.compile(args.include_db_pattern, re.IGNORECASE)
+        search_pattern = lambda vals: any(pattern.search(val) for val in vals)
+    elif args.exclude_db_pattern:
+        pattern = re.compile(args.exclude_db_pattern, re.IGNORECASE)
+        search_pattern = lambda vals: all(not pattern.search(val) for val in vals)
+    else:
+        search_pattern = None
+
+    return search_pattern
+
+
+def apply_picklist_and_pattern(db, picklist, pattern):
+    assert not (picklist and pattern)
+    if picklist:
+        db = db.select(picklist=picklist)
+    elif pattern:
+        manifest = db.manifest
+        if manifest is None:
+            error(f"ERROR on filename '{db.location}'.")
+            error("--include-db-pattern/--exclude-db-pattern require a manifest.")
+            sys.exit(-1)
+
+        manifest = manifest.filter_on_columns(pattern,
+                                              ["name", "filename", "md5"])
+        pattern_picklist = manifest.to_picklist()
+        db = db.select(picklist=pattern_picklist)
+
+    return db
+
+
 def load_query_signature(filename, ksize, select_moltype, select_md5=None):
     """Load a single signature to use as a query.
 
@@ -181,31 +284,37 @@ def traverse_find_sigs(filenames, yield_all_files=False):
 
 
 def load_dbs_and_sigs(filenames, query, is_similarity_query, *,
-                      cache_size=None, picklist=None):
+                      cache_size=None, picklist=None, pattern=None,
+                      fail_on_empty_database=False):
     """
-    Load one or more SBTs, LCAs, and/or collections of signatures.
+    Load one or more Index objects to search - databases, etc.
 
-    Check for compatibility with query.
-
-    This is basically a user-focused wrapping of _load_databases.
+    'select' on compatibility with query, and apply picklists & patterns.
     """
     query_mh = query.minhash
 
+    # set selection parameter for containment
     containment = True
     if is_similarity_query:
         containment = False
 
     databases = []
+    total_signatures_loaded = 0
+    sum_signatures_after_select = 0
     for filename in filenames:
-        notify(f'loading from {filename}...', end='\r')
+        notify(f"loading from '{filename}'...", end='\r')
 
         try:
             db = _load_database(filename, False, cache_size=cache_size)
         except ValueError as e:
             # cannot load database!
+            notify(f"ERROR on loading from '{filename}':")
             notify(str(e))
             sys.exit(-1)
 
+        total_signatures_loaded += len(db)
+
+        # get compatible signatures - moltype/ksize/num/scaled
         try:
             db = db.select(moltype=query_mh.moltype,
                            ksize=query_mh.ksize,
@@ -216,40 +325,29 @@ def load_dbs_and_sigs(filenames, query, is_similarity_query, *,
             # incompatible collection specified!
             notify(f"ERROR: cannot use '{filename}' for this query.")
             notify(str(exc))
-            sys.exit(-1)
+            if fail_on_empty_database:
+                sys.exit(-1)
+            else:
+                db = LinearIndex([])
 
         # 'select' returns nothing => all signatures filtered out. fail!
         if not db:
             notify(f"no compatible signatures found in '{filename}'")
-            sys.exit(-1)
+            if fail_on_empty_database:
+                sys.exit(-1)
 
-        if picklist:
-            db = db.select(picklist=picklist)
+        sum_signatures_after_select += len(db)
+
+        # last but not least, apply picklist!
+        db = apply_picklist_and_pattern(db, picklist, pattern)
 
         databases.append(db)
 
-    # calc num loaded info.
-    n_signatures = 0
-    n_databases = 0
-    for db in databases:
-        if db.is_database:
-            n_databases += 1
-        else:
-            n_signatures += len(db)
-
-    notify(' '*79, end='\r')
-    if n_signatures and n_databases:
-        notify(f'loaded {n_signatures} signatures and {n_databases} databases total.')
-    elif n_signatures and not n_databases:
-        notify(f'loaded {n_signatures} signatures.')
-    elif n_databases and not n_signatures:
-        notify(f'loaded {n_databases} databases.')
-
-    if databases:
-        print('')
-    else:
-        notify('** ERROR: no signatures or databases loaded?')
-        sys.exit(-1)
+    # display num loaded/num selected
+    notify("--")
+    notify(f"loaded {total_signatures_loaded} total signatures from {len(databases)} locations.")
+    notify(f"after selecting signatures compatible with search, {sum_signatures_after_select} remain.")
+    print('')
 
     return databases
 
@@ -258,11 +356,20 @@ def _load_stdin(filename, **kwargs):
     "Load collection from .sig file streamed in via stdin"
     db = None
     if filename == '-':
-        db = LinearIndex.load(sys.stdin)
+        # load as LinearIndex, then pass into MultiIndex to generate a
+        # manifest.
+        lidx = LinearIndex.load(sys.stdin, filename='-')
+        db = MultiIndex.load((lidx,), (None,), parent="-")
 
     return db
 
 
+def _load_standalone_manifest(filename, **kwargs):
+    from sourmash.index import StandaloneManifestIndex
+    idx = StandaloneManifestIndex.load(filename)
+    return idx
+
+
 def _multiindex_load_from_pathlist(filename, **kwargs):
     "Load collection from a list of signature/database files"
     db = MultiIndex.load_from_pathlist(filename)
@@ -296,19 +403,31 @@ def _load_revindex(filename, **kwargs):
     return db
 
 
+def _load_sqlite_db(filename, **kwargs):
+    return load_sqlite_index(filename)
+
+
 def _load_zipfile(filename, **kwargs):
     "Load collection from a .zip file."
     db = None
     if filename.endswith('.zip'):
         traverse_yield_all = kwargs['traverse_yield_all']
-        db = ZipFileLinearIndex.load(filename,
-                                     traverse_yield_all=traverse_yield_all)
+        try:
+            db = ZipFileLinearIndex.load(filename,
+                                         traverse_yield_all=traverse_yield_all)
+        except FileNotFoundError as exc:
+            # turn this into a ValueError => proper exception handling by
+            # _load_database.
+            raise ValueError(exc)
+
     return db
 
 
 # all loader functions, in order.
 _loader_functions = [
     ("load from stdin", _load_stdin),
+    ("load collection from sqlitedb", _load_sqlite_db),
+    ("load from standalone manifest", _load_standalone_manifest),
     ("load from path (file or directory)", _multiindex_load_from_path),
     ("load from file list", _multiindex_load_from_pathlist),
     ("load SBT", _load_sbt),
@@ -330,7 +449,7 @@ def _load_database(filename, traverse_yield_all, *, cache_size=None):
     # but nothing else.
     for n, (desc, load_fn) in enumerate(_loader_functions):
         try:
-            debug_literal(f"_load_databases: trying loader fn {n} {desc}")
+            debug_literal(f"_load_databases: trying loader fn {n} '{desc}'")
             db = load_fn(filename,
                          traverse_yield_all=traverse_yield_all,
                          cache_size=cache_size)
@@ -340,6 +459,7 @@ def _load_database(filename, traverse_yield_all, *, cache_size=None):
 
         if db is not None:
             loaded = True
+            debug_literal("_load_databases: success!")
             break
 
     # check to see if it's a FASTA/FASTQ record (i.e. screed loadable)
@@ -390,6 +510,7 @@ def load_file_as_signatures(filename, *, select_moltype=None, ksize=None,
                             picklist=None,
                             yield_all_files=False,
                             progress=None,
+                            pattern=None,
                             _use_manifest=True):
     """Load 'filename' as a collection of signatures. Return an iterable.
 
@@ -406,6 +527,8 @@ def load_file_as_signatures(filename, *, select_moltype=None, ksize=None,
     yield_all_files=True, will attempt to load all files.
 
     Applies selector function if select_moltype, ksize or picklist are given.
+
+    'pattern' is a function that returns True on matching values.
     """
     if progress:
         progress.notify(filename)
@@ -416,7 +539,11 @@ def load_file_as_signatures(filename, *, select_moltype=None, ksize=None,
     if not _use_manifest and db.manifest:
         db.manifest = None
 
-    db = db.select(moltype=select_moltype, ksize=ksize, picklist=picklist)
+    db = db.select(moltype=select_moltype, ksize=ksize)
+
+    # apply pattern search & picklist
+    db = apply_picklist_and_pattern(db, picklist, pattern)
+
     loader = db.signatures()
 
     if progress is not None:
@@ -445,7 +572,7 @@ def load_pathlist_from_file(filename):
     return file_list
 
 
-class FileOutput(object):
+class FileOutput:
     """A context manager for file outputs that handles sys.stdout gracefully.
 
     Usage:
@@ -481,7 +608,8 @@ class FileOutput(object):
         return self.fp
 
     def close(self):
-        self.fp.close()
+        if self.fp is not None: # in case of stdout
+            self.fp.close()
 
     def __enter__(self):
         return self.open()
@@ -493,6 +621,7 @@ class FileOutput(object):
 
         return False
 
+
 class FileOutputCSV(FileOutput):
     """A context manager for CSV file outputs.
 
@@ -521,11 +650,123 @@ class FileOutputCSV(FileOutput):
     def open(self):
         if self.filename == '-' or self.filename is None:
             return sys.stdout
-        self.fp = open(self.filename, 'w', newline='')
+        if self.filename.endswith('.gz'):
+            self.fp = gzip.open(self.filename, 'wt', newline='')
+        else:
+            self.fp = open(self.filename, 'w', newline='')
         return self.fp
 
 
-class SignatureLoadingProgress(object):
+class _DictReader_with_version:
+    """A version of csv.DictReader that allows a comment line with a version,
+    e.g.
+
+    # SOURMASH-MANIFEST-VERSION: 1.0
+
+    The version is stored as a 2-tuple in the 'version_info' attribute.
+    """
+    def __init__(self, textfp, *, delimiter=','):
+        self.version_info = []
+
+        # is there a '#' in the raw buffer pos 0?
+        ch = textfp.buffer.peek(1)
+
+        try:
+            ch = ch.decode('utf-8')
+        except UnicodeDecodeError:
+            raise csv.Error("unable to read CSV file")
+
+        # yes - read a line from the text buffer => parse
+        if ch.startswith('#'):
+            line = textfp.readline()
+            assert line.startswith('# '), line
+
+            # note, this can set version_info to lots of different things.
+            # revisit later, I guess. CTB.
+            self.version_info = line[2:].strip().split(': ', 2)
+
+        # build a DictReader from the remaining stream
+        self.reader = csv.DictReader(textfp, delimiter=delimiter)
+        self.fieldnames = self.reader.fieldnames
+
+    def __iter__(self):
+        for row in self.reader:
+            yield row
+
+
+@contextlib.contextmanager
+def FileInputCSV(filename, *, encoding='utf-8', default_csv_name=None,
+                 zipfile_obj=None, delimiter=','):
+    """A context manager for reading in CSV files in gzip, zip or text format.
+
+    Assumes comma delimiter, and uses csv.DictReader.
+
+    Note: does not support stdin.
+
+    Note: it seems surprisingly hard to write code that generically handles
+    any file handle being passed in; the manifest loading code, in particular,
+    uses ZipStorage.load => StringIO obj, which doesn't support peek etc.
+    So for now, this context manager is focused on situations where it owns
+    the file handle (opens/closes the file).
+    """
+    fp = None
+
+    if zipfile_obj and not default_csv_name:
+        raise ValueError("must provide default_csv_name with a zipfile_obj")
+
+    # first, try to load 'default_csv_name' from a zipfile:
+    if default_csv_name:
+        # were we given a zipfile obj?
+        if zipfile_obj:
+            try:
+                zi = zipfile_obj.getinfo(default_csv_name)
+                with zipfile_obj.open(zi) as fp:
+                    textfp = TextIOWrapper(fp,
+                                           encoding=encoding,
+                                           newline="")
+                    r = _DictReader_with_version(textfp, delimiter=delimiter)
+                    yield r
+            except (zipfile.BadZipFile, KeyError):
+                pass # uh oh, we were given a zipfile_obj and it FAILED.
+
+            # no matter what, if given zipfile_obj don't try .gz or regular csv
+            return
+        else:
+            try:
+                with zipfile.ZipFile(filename, 'r') as zip_fp:
+                    zi = zip_fp.getinfo(default_csv_name)
+                    with zip_fp.open(zi) as fp:
+                        textfp = TextIOWrapper(fp,
+                                               encoding=encoding,
+                                               newline="")
+                        r = _DictReader_with_version(textfp, delimiter=delimiter)
+                        yield r
+
+                # if we got this far with no exceptions, we found
+                # the CSV in the zip file. exit generator!
+                return
+            except (zipfile.BadZipFile, KeyError):
+                # no zipfile_obj => it's ok to continue onwards to .gz
+                # and regular CSV.
+                pass
+
+    # ok, not a zip file - try .gz:
+    try:
+        with gzip.open(filename, "rt", newline="", encoding=encoding) as fp:
+            fp.buffer.peek(1)          # force exception if not a gzip file
+            r = _DictReader_with_version(fp, delimiter=delimiter)
+            yield r
+        return
+    except gzip.BadGzipFile:
+        pass
+
+    # neither zip nor gz; regular file!
+    with open(filename, 'rt', newline="", encoding=encoding) as fp:
+        r = _DictReader_with_version(fp, delimiter=delimiter)
+        yield r
+
+
+class SignatureLoadingProgress:
     """A wrapper for signature loading progress reporting.
 
     Instantiate this class once, and then pass it to load_file_as_signatures
@@ -589,7 +830,8 @@ class SignatureLoadingProgress(object):
 
 
 def load_many_signatures(locations, progress, *, yield_all_files=False,
-                         ksize=None, moltype=None, picklist=None, force=False):
+                         ksize=None, moltype=None, picklist=None, force=False,
+                         pattern=None):
     """
     Load many signatures from multiple files, with progress indicators.
 
@@ -606,9 +848,9 @@ def load_many_signatures(locations, progress, *, yield_all_files=False,
         try:
             # open index,
             idx = load_file_as_index(loc, yield_all_files=yield_all_files)
+            idx = idx.select(ksize=ksize, moltype=moltype)
 
-            # select on parameters as desired,
-            idx = idx.select(ksize=ksize, moltype=moltype, picklist=picklist)
+            idx = apply_picklist_and_pattern(idx, picklist, pattern)
 
             # start up iterator,
             loader = idx.signatures_with_location()
@@ -622,11 +864,11 @@ def load_many_signatures(locations, progress, *, yield_all_files=False,
         except ValueError as exc:
             # trap expected errors, and either power through or display + exit.
             if force:
-                notify("ERROR: {}", str(exc))
+                notify(f"ERROR: {str(exc)}")
                 notify("(continuing)")
                 continue
             else:
-                notify("ERROR: {}", str(exc))
+                notify(f"ERROR: {str(exc)}")
                 sys.exit(-1)
         except KeyboardInterrupt:
             notify("Received CTRL-C - exiting.")
@@ -636,10 +878,64 @@ def load_many_signatures(locations, progress, *, yield_all_files=False,
     notify(f"loaded {len(progress)} signatures total, from {n_files} files")
 
 
+def get_manifest(idx, *, require=True, rebuild=False):
+    """
+    Retrieve a manifest for this idx, loaded with `load_file_as_index`.
+
+    Even if a manifest exists and `rebuild` is True, rebuild the manifest.
+    If a manifest does not exist or `rebuild` is True, try to build one.
+    If a manifest cannot be built and `require` is True, error exit.
+
+    In the case where `require=False` and a manifest cannot be built,
+    may return None. Otherwise always returns a manifest.
+    """
+    from sourmash.index import CollectionManifest
+
+    m = idx.manifest
+
+    # has one, and don't want to rebuild? easy! return!
+    if m is not None and not rebuild:
+        debug_literal("get_manifest: found manifest")
+        return m
+
+    debug_literal(f"get_manifest: no manifest found / rebuild={rebuild}")
+
+    # need to build one...
+    try:
+        notify("Generating a manifest...")
+        m = CollectionManifest.create_manifest(idx._signatures_with_internal(),
+                                               include_signature=False)
+        debug_literal("get_manifest: rebuilt manifest.")
+    except NotImplementedError:
+        if require:
+            error(f"ERROR: manifests cannot be generated for {idx.location}")
+            sys.exit(-1)
+        else:
+            debug_literal("get_manifest: cannot build manifest, not req'd")
+            return None
+
+    return m
+
 #
 # enum and classes for saving signatures progressively
 #
 
+def _get_signatures_from_rust(siglist):
+    for ss in siglist:
+        try:
+            ss.md5sum()
+            yield ss
+        except sourmash.exceptions.Panic:
+            # this deals with a disconnect between the way Rust
+            # and Python handle signatures; Python expects one
+            # minhash (and hence one md5sum) per signature, while
+            # Rust supports multiple. For now, go through serializing
+            # and deserializing the signature! See issue #1167 for more.
+            json_str = sourmash.save_signatures([ss])
+            for ss in sourmash.load_signatures(json_str):
+                yield ss
+
+
 class _BaseSaveSignaturesToLocation:
     "Base signature saving class. Track location (if any) and count."
     def __init__(self, location):
@@ -698,8 +994,7 @@ class SaveSignatures_Directory(_BaseSaveSignaturesToLocation):
         except FileExistsError:
             pass
         except:
-            notify("ERROR: cannot create signature output directory '{}'",
-                   self.location)
+            notify(f"ERROR: cannot create signature output directory '{self.location}'")
             sys.exit(-1)
 
     def add(self, ss):
@@ -720,8 +1015,38 @@ class SaveSignatures_Directory(_BaseSaveSignaturesToLocation):
             sigmod.save_signatures([ss], fp, compression=1)
 
 
-class SaveSignatures_SigFile(_BaseSaveSignaturesToLocation):
+class SaveSignatures_SqliteIndex(_BaseSaveSignaturesToLocation):
     "Save signatures within a directory, using md5sum names."
+    def __init__(self, location):
+        super().__init__(location)
+        self.location = location
+        self.idx = None
+        self.cursor = None
+
+    def __repr__(self):
+        return f"SaveSignatures_SqliteIndex('{self.location}')"
+
+    def close(self):
+        self.idx.commit()
+        self.cursor.execute('VACUUM')
+        self.idx.close()
+
+    def open(self):
+        self.idx = SqliteIndex.create(self.location, append=True)
+        self.cursor = self.idx.cursor()
+
+    def add(self, add_sig):
+        for ss in _get_signatures_from_rust([add_sig]):
+            super().add(ss)
+            self.idx.insert(ss, cursor=self.cursor, commit=False)
+
+            # commit every 1000 signatures.
+            if self.count % 1000 == 0:
+                self.idx.commit()
+
+
+class SaveSignatures_SigFile(_BaseSaveSignaturesToLocation):
+    "Save signatures to a .sig JSON file."
     def __init__(self, location):
         super().__init__(location)
         self.keep = []
@@ -761,7 +1086,7 @@ class SaveSignatures_ZipFile(_BaseSaveSignaturesToLocation):
     "Save compressed signatures in an uncompressed Zip file."
     def __init__(self, location):
         super().__init__(location)
-        self.zf = None
+        self.storage = None
 
     def __repr__(self):
         return f"SaveSignatures_ZipFile('{self.location}')"
@@ -775,70 +1100,88 @@ class SaveSignatures_ZipFile(_BaseSaveSignaturesToLocation):
         manifest.write_to_csv(manifest_fp, write_header=True)
         manifest_data = manifest_fp.getvalue().encode("utf-8")
 
-        # compress the manifest --
-        self.zf.writestr(manifest_name, manifest_data,
-                         compress_type=zipfile.ZIP_DEFLATED)
-
-        # set permissions:
-        zi = self.zf.getinfo(manifest_name)
-        zi.external_attr = 0o444 << 16 # give a+r access
-
-        self.zf.close()
+        self.storage.save(manifest_name, manifest_data, overwrite=True,
+                          compress=True)
+        self.storage.flush()
+        self.storage.close()
 
     def open(self):
-        self.zf = zipfile.ZipFile(self.location, 'w', zipfile.ZIP_STORED)
-        self.manifest_rows = []
+        from .sbt_storage import ZipStorage
+
+        do_create = True
+        if os.path.exists(self.location):
+            do_create = False
+
+        storage = None
+        try:
+            storage = ZipStorage(self.location, mode="w")
+        except zipfile.BadZipFile:
+            pass
+
+        if storage is None:
+            raise ValueError(f"File '{self.location}' cannot be opened as a zip file.")
+
+        if not storage.subdir:
+            storage.subdir = 'signatures'
+
+        # now, try to load manifest
+        try:
+            manifest_data = storage.load('SOURMASH-MANIFEST.csv')
+        except (FileNotFoundError, KeyError):
+            # if file already exists must have manifest...
+            if not do_create:
+                raise ValueError(f"Cannot add to existing zipfile '{self.location}' without a manifest")
+            self.manifest_rows = []
+        else:
+            # success! decode manifest_data, create manifest rows => append.
+            manifest_data = manifest_data.decode('utf-8')
+            manifest_fp = StringIO(manifest_data)
+            manifest = CollectionManifest.load_from_csv(manifest_fp)
+            self.manifest_rows = list(manifest._select())
+
+        self.storage = storage
 
     def _exists(self, name):
         try:
-            self.zf.getinfo(name)
+            self.storage.load(name)
             return True
         except KeyError:
             return False
 
-    def add(self, ss):
-        if not self.zf:
+    def add(self, add_sig):
+        if not self.storage:
             raise ValueError("this output is not open")
-        super().add(ss)
 
-        md5 = ss.md5sum()
-        outname = f"signatures/{md5}.sig.gz"
+        for ss in _get_signatures_from_rust([add_sig]):
+            buf = sigmod.save_signatures([ss], compression=1)
+            md5 = ss.md5sum()
 
-        # don't overwrite even if duplicate md5sum.
-        if self._exists(outname):
-            i = 0
-            while 1:
-                outname = os.path.join(self.location, f"{md5}_{i}.sig.gz")
-                if not self._exists(outname):
-                    break
-                i += 1
+            storage = self.storage
+            path = f'{storage.subdir}/{md5}.sig.gz'
+            location = storage.save(path, buf)
 
-        json_str = sourmash.save_signatures([ss], compression=1)
-        self.zf.writestr(outname, json_str)
-
-        # set permissions:
-        zi = self.zf.getinfo(outname)
-        zi.external_attr = 0o444 << 16 # give a+r access
-
-        # update manifest
-        row = CollectionManifest.make_manifest_row(ss, outname,
-                                                   include_signature=False)
-        self.manifest_rows.append(row)
+            # update manifest
+            row = CollectionManifest.make_manifest_row(ss, location,
+                                                       include_signature=False)
+            self.manifest_rows.append(row)
+            super().add(ss)
 
 
 class SigFileSaveType(Enum):
+    NO_OUTPUT = 0
     SIGFILE = 1
     SIGFILE_GZ = 2
     DIRECTORY = 3
     ZIPFILE = 4
-    NO_OUTPUT = 5
+    SQLITEDB = 5
 
 _save_classes = {
+    SigFileSaveType.NO_OUTPUT: SaveSignatures_NoOutput,
     SigFileSaveType.SIGFILE: SaveSignatures_SigFile,
     SigFileSaveType.SIGFILE_GZ: SaveSignatures_SigFile,
     SigFileSaveType.DIRECTORY: SaveSignatures_Directory,
     SigFileSaveType.ZIPFILE: SaveSignatures_ZipFile,
-    SigFileSaveType.NO_OUTPUT: SaveSignatures_NoOutput
+    SigFileSaveType.SQLITEDB: SaveSignatures_SqliteIndex,
 }
 
 
@@ -855,6 +1198,8 @@ def SaveSignaturesToLocation(filename, *, force_type=None):
             save_type = SigFileSaveType.SIGFILE_GZ
         elif filename.endswith('.zip'):
             save_type = SigFileSaveType.ZIPFILE
+        elif filename.endswith('.sqldb'):
+            save_type = SigFileSaveType.SQLITEDB
         else:
             # default to SIGFILE intentionally!
             save_type = SigFileSaveType.SIGFILE
